@@ -1,19 +1,20 @@
 #!/usr/bin/env python
 
-"""Align two POGs"""
+"""Align two Alignables, each of which can be a sequence or a subalignment
+produced by the same code."""
 
 # How many cells before using linear space alignment algorithm.
-# Should probably set to about physical memory / 8
-HIRSCHBERG_LIMIT = 25000000
+# Should probably set to about half of physical memory / PointerEncoder.bytes
+HIRSCHBERG_LIMIT = 10**8
 
 import numpy
 
-numpy.seterr(all='ignore')
-
-Float = numpy.core.numerictypes.sctype2char(float)
-Int = numpy.core.numerictypes.sctype2char(int)
-dot = numpy.dot
+# setting global state on module load is bad practice and can be ineffective,
+# and this setting matches the defaults anyway, but left here as reference 
+# in case it needs to be put back in a more runtime way.
+#numpy.seterr(all='ignore')
 
+import warnings
 import logging
 LOG = logging.getLogger('cogent.align')
 
@@ -27,12 +28,12 @@ from cogent.align.traceback import map_traceback
 from cogent.util.modules import importVersionedModule, ExpectedImportError
 try:
     pyrex_align_module = importVersionedModule('_pairwise_pogs', globals(),
-            (2, 0), LOG, "slow Python alignment implementation")
+            (3, 0), LOG, "slow Python alignment implementation")
 except ExpectedImportError:
     pyrex_align_module = None
 try:
     pyrex_seq_align_module = importVersionedModule('_pairwise_seqs', globals(),
-            (2, 1), LOG, "slow Python alignment implementation")
+            (3, 0), LOG, "slow Python alignment implementation")
 except ExpectedImportError:
     pyrex_seq_align_module = None
 
@@ -40,25 +41,51 @@ __author__ = "Peter Maxwell"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Peter Maxwell"
 __email__ = "pm67nz@gmail.com"
 __status__ = "Production"
 
-def encode_state(dx, dy, state):
-    # Pack 3 small ints into a long to save memory but also avoid
-    # using numpy.Int8, because that complicates the numerical
-    # pyrex stuff and there is no need to look up the 3 values
-    # separately anyway
-    return (dx*256+dy)*256+state
+class PointerEncoding(object):
+    """Pack very small ints into a byte.  The last field, state, is assigned
+    whatever bits are left over after the x and y pointers have claimed what 
+    they need, which is expected to be only 2 bits each at most"""
+    
+    dtype = numpy.int8
+    bytes = 1
+
+    def __init__(self, x, y):
+        assert x > 0 and y > 0, (x,y)
+        (x, y) = (numpy.ceil(numpy.log2([x+1,y+1]))).astype(int)
+        s = 8 * self.bytes - sum([x, y])
+        assert s**2 >= 4+1, (x,y,s) # min states required
+        self.widths = numpy.array([x,y,s]).astype(int)
+        self.limits = 2 ** self.widths
+        self.max_states = self.limits[-1]
+        if DEBUG:
+            print self.max_states, "states allowed in viterbi traceback"
+        self.positions = numpy.array([0, x, x+y], int)
+        #a.flags.writeable = False
+    def encode(self, x, y, s):
+        parts = numpy.asarray([x, y, s], int)
+        assert all(parts < self.limits), (parts, self.limits)
+        return (parts << self.positions).sum()
+    def decode(self, coded):
+        return (coded >> self.positions) % self.limits
+    def getEmptyArray(self, shape):
+        return numpy.zeros(shape, self.dtype)
+    
+DEBUG = False
 
 def py_calc_rows(plan, x_index, y_index, i_low, i_high, j_low, j_high,
         preds, state_directions, T,
-        xgap_scores, ygap_scores, match_scores, rows, track, viterbi,
-        local=False, use_scaling=False, use_logs=False):
-    """python function for POG calculations"""
-    # XXX assert not use_scaling
-    DEBUG = False
+        xgap_scores, ygap_scores, match_scores, rows, track, track_enc, 
+        viterbi, local=False, use_scaling=False, use_logs=False):
+    """Pure python version of the dynamic programming algorithms
+    Forward and Viterbi.  Works on sequences and POGs.  Unli"""
+    if use_scaling:
+        warnings.warn("Pure python version of DP code can suffer underflows")
+        # because it ignores 'exponents', the Pyrex version doesn't.
     source_states = range(len(T))
     BEGIN = 0
     ERROR = len(T)
@@ -82,12 +109,11 @@ def py_calc_rows(plan, x_index, y_index, i_low, i_high, j_low, j_high,
             y = y_index[j]
             j_sources = preds[1][j]
             for (state, bin, dx, dy) in state_directions:
-                #print (i, j, state, bin, dx, dy)
                 if (local and dx and dy):
                     cumulative_score = T[BEGIN, state]
                     pointer = (dx, dy, BEGIN)
                 else:
-                    cumulative_score = 0.0
+                    cumulative_score = impossible
                     pointer = (0, 0, ERROR)
                 for (a, prev_i) in enumerate([[i], i_sources][dx]):
                     source_row = rows[plan[prev_i]]
@@ -101,6 +127,8 @@ def py_calc_rows(plan, x_index, y_index, i_low, i_high, j_low, j_high,
                                     candidate = prev_value + transition
                                 else:
                                     candidate = prev_value * transition
+                                #if DEBUG:
+                                #    print prev_state, prev_value, state
                                 if candidate > cumulative_score:
                                     cumulative_score = candidate
                                     pointer = (a+dx, b+dy, prev_state)
@@ -114,21 +142,56 @@ def py_calc_rows(plan, x_index, y_index, i_low, i_high, j_low, j_high,
                     d_score = ygap_scores[bin, y]
                 else:
                     d_score = neutral_score
+                #if DEBUG:
+                #    print (dx, dy), d_score, cumulative_score
                 if use_logs:
                     current_row[j, state] = cumulative_score + d_score
                 else:
                     current_row[j, state] = cumulative_score * d_score
-                if DEBUG:
-                    print (i, j), state, ':', cumulative_score, '*', d_score, '=', \
-                            current_row[j, state]
                 if track is not None:
-                    track[i,j,state] = encode_state(*pointer)
+                    track[i,j,state] = (numpy.array(pointer) << track_enc).sum()
                 if (i==i_high-1 and j==j_high-1 and not local) or (
                         local and dx and dy and current_row[j, state] > best_score):
                     (best, best_score) = (((i, j), state), current_row[j, state])
-    #print 'best_score, result', best_score, best + (numpy.log(best_score),)
-    return best + (numpy.log(best_score),)
+    #if DEBUG:
+    #    print i_low, i_high, j_low, j_high
+    #    print 'best_score %5.1f  at  %s' % (numpy.log(best_score), best)
+    if not use_logs:
+        best_score = numpy.log(best_score)
+    return best + (best_score,)
 
+class TrackBack(object):
+    def __init__(self, tlist):
+        self.tlist = tlist
+    
+    def __str__(self):
+        return ''.join('(%s,%s)%s' % 
+            (x,y, '.xym'[dx+2*dy]) for (state, (x,y), (dx,dy))
+            in self.tlist)
+            
+    def offset(self, X, Y):
+        tlist = [(state, (x+X, y+Y), dxy) for (state, (x,y), dxy) in self.tlist]
+        return TrackBack(tlist)
+                
+    def __add__(self, other):
+        return TrackBack(self.tlist + other.tlist)
+        
+    #def asStatePosnTransTuples(self):
+    #    return iter(self.tlist)
+        
+    def asStatePosnTuples(self):
+        return [(s,p) for (s,p,d) in self.tlist]
+
+    def asBinPosTuples(self, state_directions):
+        bin_map = dict((state, bin) for (state, bin, dx, dy) in
+            state_directions)
+        result = []
+        for (state, posn, (dx, dy)) in self.tlist:
+            pos = [[None, i-1][d] for (i,d) in zip(
+                    posn, [dx, dy])]
+            result.append((bin_map.get(int(state), None), pos))
+        return result
+    
 
 class Pair(object):
     def __init__(self, alignable1, alignable2, backward=False):
@@ -158,7 +221,9 @@ class Pair(object):
             alignables = [a.backward() for a in alignables]
         
         self.children = [alignable1, alignable2] = alignables
-        self.max_preds = max(alignable.max_preds for alignable in alignables)
+        self.max_preds = [alignable.max_preds for alignable in alignables]
+        self.pointer_encoding = PointerEncoding(*self.max_preds)
+
         self.size = [len(alignable1), len(alignable2)]
         self.uniq_size = [len(alignable1.plh), len(alignable2.plh)]
         self.plan = numpy.array(alignable1.getRowAssignmentPlan())
@@ -190,32 +255,29 @@ class Pair(object):
         children = [child[dim_index] for (child, dim_index) in zip(
                 self.children, index)]
         return Pair(*children)
-    
-    def penultimateRows(self):
-        return self.children[0].penultimateRows()
-    
-    def _decode_state(self, track, posn, pstate):
+        
+    def _decode_state(self, track, encoding, posn, pstate):
         coded = int(track[posn[0], posn[1], pstate])
-        (coded, state) = divmod(coded, 256)
+        (a, b, state) = encoding.decode(coded)
         if state >= track.shape[-1]:
             raise ArithmeticError('Error state in traceback')
-        (dx, dy) = divmod(coded, 256)
         (x, y) = posn
         if state == -1:
             next = (x, y)
         else:
-            if dx: x = self.children[0][x][dx-1]
-            if dy: y = self.children[1][y][dy-1]
-            next = numpy.array([x,y], Int)
-        return (next, (dx, dy), state)
+            if a: x = self.children[0][x][a-1]
+            if b: y = self.children[1][y][b-1]
+            next = numpy.array([x,y], int)
+        return (next, (a, b), state)
     
-    def traceback(self, track, posn, state, skip_last=False):
+    def traceback(self, track, encoding, posn, state, skip_last=False):
         result = []
         started = False
         while 1:
-            (nposn, (dx, dy), nstate) = self._decode_state(track, posn, state)
+            (nposn, (a, b), nstate) = self._decode_state(track, encoding, 
+                    posn, state)
             if state:
-                result.append((state, posn, (dx, dy)))
+                result.append((state, posn, (a>0, b>0)))
             if started and state == 0:
                 break
             (posn, state) = (nposn, nstate)
@@ -223,7 +285,7 @@ class Pair(object):
         result.reverse()
         if skip_last:
             result.pop()
-        return result
+        return TrackBack(result)
     
     def edge2plh(self, edge, plhs):
         bins = plhs[0].shape[0]
@@ -234,12 +296,12 @@ class Pair(object):
     def getPOG(self, aligned_positions):
         (pog1, pog2) = [child.getPOG() for child in self.children]
         return pog1.traceback(pog2, aligned_positions)
-    
-    def getEmptyStateArray(self, n_states):
-        assert n_states < 255
-        assert self.max_preds < 255, ("\n\n", n_states, "\n\n", self.max_preds)
-        return numpy.zeros(self.size + [n_states], int) #, numpy.Int8)
-    
+        
+    def getPointerEncoding(self, n_states):
+        assert n_states <= self.pointer_encoding.max_states, (
+            n_states, self.pointer_encoding.max_states)
+        return self.pointer_encoding
+
     def getScoreArraysShape(self):
         needed = max(self.plan) + 1
         N = self.size[1]
@@ -247,21 +309,24 @@ class Pair(object):
     
     def getEmptyScoreArrays(self, n_states, dp_options):
         shape = self.getScoreArraysShape() + (n_states,)
-        mantissas = numpy.zeros(shape, Float)
+        mantissas = numpy.zeros(shape, float)
         if dp_options.use_logs:
-            mantissas[:] = numpylog(0.0)
+            mantissas[:] = numpy.log(0.0)
         if dp_options.use_scaling:
-            exponents = numpy.ones(shape, Int) * -10000
+            exponents = numpy.ones(shape, int) * -10000
         else:
             exponents = None
         return (mantissas, exponents)
     
     def calcRows(self, i_low, i_high, j_low, j_high, state_directions,
-            T, scores, rows, track, viterbi, **kw):
+            T, scores, rows, track, track_encoding, viterbi, **kw):
         (match_scores, (xscores, yscores)) = scores
+        track_enc = track_encoding and track_encoding.positions
+        #print T
         return self.aligner(self.plan, self.x_index, self.y_index,
                 i_low, i_high, j_low, j_high, self.children, state_directions,
-                T, xscores, yscores, match_scores, rows, track, viterbi, **kw)
+                T, xscores, yscores, match_scores, rows, track, 
+                track_enc, viterbi, **kw)
     
 
 class _Alignable(object):
@@ -270,10 +335,9 @@ class _Alignable(object):
         self.alphabet = leaf.alphabet
         (uniq, alphabet_size) = leaf.input_likelihoods.shape
         full = len(leaf.index)
-        self.plh = numpy.zeros([uniq+2, alphabet_size], Float)
+        self.plh = numpy.zeros([uniq+2, alphabet_size], float)
         self.plh[1:-1] = leaf.input_likelihoods
         self.index = numpy.zeros([full+2], int)
-        #print leaf.shape, self.index.shape, leaf.index.shape
         self.index[1:-1] = numpy.asarray(leaf.index) + 1
         self.index[0] = 0
         self.index[full+1] = uniq+1
@@ -296,9 +360,6 @@ class _Alignable(object):
             self._combined = self._asCombinedArray()
         return self._combined
     
-    def penultimateRows(self):
-        return sorted(self[len(self)-1])
-    
     def getRowAssignmentPlan(self):
         d = self.getOuterLoopDiscardPoints()
         free = set()
@@ -317,6 +378,8 @@ class _Alignable(object):
     
 
 class AlignablePOG(_Alignable):
+    """Alignable wrapper of a Partial Object Graph, ie: subalignment"""
+    
     def __init__(self, leaf, pog, children=None):
         assert len(leaf) == len(pog), (len(leaf), len(pog))
         _Alignable.__init__(self, leaf)
@@ -325,6 +388,10 @@ class AlignablePOG(_Alignable):
         self.pog = pog
         if children is not None:
             self.aligneds = self._calcAligneds(children)
+        self.leaf = leaf
+    
+    def __repr__(self):
+        return 'AlPOG(%s,%s)' % (self.pog.all_jumps, repr(self.leaf))
     
     def getAlignment(self):
         return LoadSeqs(data=self.aligneds)
@@ -361,6 +428,9 @@ class AlignablePOG(_Alignable):
             leaf = self.leaf[index]
             return AlignablePOG(leaf, pog)
     
+    def midlinks(self):
+        return self.pog.midlinks()
+    
     def getOuterLoopDiscardPoints(self):
         # for score row caching
         last_successor = {}
@@ -375,6 +445,9 @@ class AlignablePOG(_Alignable):
     
 
 class AlignableSeq(_Alignable):
+    """Wrapper for a Sequence which gives it the same interface as an
+    AlignablePOG"""
+    
     def __init__(self, leaf):
         _Alignable.__init__(self, leaf)
         if hasattr(leaf, 'sequence'):
@@ -384,6 +457,9 @@ class AlignableSeq(_Alignable):
         self.max_preds = 1
         self._pog = None
     
+    def __repr__(self):
+        return 'AlSeq(%s)' % (getattr(self, 'seq', '?'))
+    
     def getPOG(self):
         if self._pog is None:
             self._pog = leaf2pog(self.leaf)
@@ -414,7 +490,11 @@ class AlignableSeq(_Alignable):
         #    return self
         else:
             return AlignableSeq(self.leaf[index])
-    
+
+    def midlinks(self):
+        half = len(self.leaf) // 2
+        return  [(half, half)]
+
     def getOuterLoopDiscardPoints(self):
         return [[]] + [[i] for i in range(len(self)-1)]
     
@@ -433,7 +513,7 @@ def adaptPairTM(pairTM, finite=False):
         pairTM = pairTM.withoutSilentStates()
         stationary_probs = numpy.array(pairTM.StationaryProbs)
         T = pairTM.Matrix
-        full_matrix = numpy.zeros([len(T)+2, len(T)+2], Float)
+        full_matrix = numpy.zeros([len(T)+2, len(T)+2], float)
         full_matrix[1:-1,1:-1] = T
         full_matrix[0,1:-1] = stationary_probs # from BEGIN
         full_matrix[:,-1] = 1.0  #  to END
@@ -444,8 +524,7 @@ def adaptPairTM(pairTM, finite=False):
     state_directions_list.sort(key=this_row_last)
     # sorting into desirable order (sort may not be necessary)
     
-    state_directions = numpy.zeros([len(state_directions_list), 4],
-            Int)
+    state_directions = numpy.zeros([len(state_directions_list), 4], int)
     for (i, (state, emit)) in enumerate(state_directions_list):
         (dx, dy) = emit
         assert dx==0 or dy==0 or dx==dy
@@ -462,17 +541,17 @@ class PairEmissionProbs(object):
         self.scores = {}
     
     def makePartialLikelihoods(self, use_cost_function):
-        # use_cost_function specifies whether eqn 2 of LG invoked
+        # use_cost_function specifies whether eqn 2 of Loytynoja & Goldman 
+        # is applied.  Without it insertions may be favored over deletions
+        # because the emission probs of the insert aren't counted.
         plhs = [[], []]
         gap_plhs = [[], []]
         for bin in self.bins:
             for (dim, pred) in enumerate(self.pair.children):
                 # first and last should be special START and END nodes
-                # evolve
                 plh = numpy.inner(pred.plh, bin.ppsubs[dim])
                 gap_plh = numpy.inner(pred.plh, bin.mprobs)
                 if use_cost_function:
-                    # is this correct?
                     plh /= gap_plh[..., numpy.newaxis]
                     gap_plh[:] = 1.0
                 else:
@@ -482,49 +561,46 @@ class PairEmissionProbs(object):
         for dim in [0,1]:
             plhs[dim] = numpy.array(plhs[dim])
             gap_plhs[dim] = numpy.array(gap_plhs[dim])
-            #print 'a', plhs[dim].shape
         return (plhs, gap_plhs)
     
     def _makeEmissionProbs(self, use_cost_function):
         (plhs, gap_scores) = self.makePartialLikelihoods(use_cost_function)
         match_scores = numpy.zeros([len(self.bins)] + self.pair.uniq_size,
-                                    Float)
+                                    float)
         for (b, (x, y, bin)) in enumerate(zip(plhs[0], plhs[1], self.bins)):
-            if numpy.__version__.startswith('24.'):
-                # this should probably be removed on numpy port
-                # innerproduct failing at vecLib layer - Numeric bug #1369004
-                match_scores[b] = dot(x*bin.mprobs, numpy.transpose(y))
-            else:
-                match_scores[b] = numpy.inner(x*bin.mprobs, y)
+            match_scores[b] = numpy.inner(x*bin.mprobs, y)
         match_scores[:, 0, 0] = match_scores[:, -1, -1] = 1.0
         return (match_scores, gap_scores)
     
-    def _getEmissionProbs(self, use_logs=False, use_cost_function=True):
+    def _getEmissionProbs(self, use_logs, use_cost_function):
         key = (use_logs, use_cost_function)
         if key not in self.scores:
             if use_logs:
-                (M, (X, Y)) = self._getEmissionProbs(
-                        use_logs=False, use_cost_function=use_cost_function)
+                (M, (X, Y)) = self._getEmissionProbs(False, use_cost_function)
                 (M, X, Y) = [numpy.log(a) for a in [M, X, Y]]
                 self.scores[key] = (M, (X, Y))
             else:
-                self.scores[key] = self._makeEmissionProbs(
-                        use_cost_function=use_cost_function)
+                self.scores[key] = self._makeEmissionProbs(use_cost_function)
         return self.scores[key]
     
     def _calc_global_probs(self, pair, scores, kw, state_directions,
-            T, rows, tb, backward=False):
+            T, rows, cells, backward=False):
+        if kw['use_logs']:
+            (impossible, inevitable) = (-numpy.inf, 0.0)
+        else:
+            (impossible, inevitable) = (0.0, 1.0)
         (M, N) = pair.size
         (mantissas, exponents) = rows
-        mantissas[0,0,0] = 1.0
-        exponents[0,0,0] = 0
+        mantissas[0,0,0] = inevitable
+        if exponents is not None:
+            exponents[0,0,0] = 0
         probs = []
         last_i = -1
         to_end = numpy.array([(len(T)-1, 0, 0, 0)])
-        for (state, (i,j), dxy) in tb:
+        for (state, (i,j)) in cells:
             if i > last_i:
                 rr = pair.calcRows(last_i+1, i+1, 0, N-1,
-                    state_directions, T, scores, rows, None, **kw)
+                    state_directions, T, scores, rows, None, None, **kw)
             else:
                 assert i == last_i, (i, last_i)
             last_i = i
@@ -532,33 +608,21 @@ class PairEmissionProbs(object):
             if backward:
                 T2[:, -1] = T[:, state]
             else:
-                T2[:, -1] = 0.0
-                T2[state, -1] = 1.0
+                T2[:, -1] = impossible
+                T2[state, -1] = inevitable
+            global DEBUG
+            _d = DEBUG
+            DEBUG = False
             (maxpos, state, score) = pair.calcRows(
-                    i, i+1, j, j+1, to_end, T2, scores, rows, None, **kw)
+                    i, i+1, j, j+1, to_end, T2, scores, rows, None, None, **kw)
+            DEBUG = _d
             probs.append(score)
         return numpy.array(probs)
-    
-    def _calc_global(self, pair, scores, kw, state_directions, T,
-            rows, track):
-        (M, N) = pair.size
-        pair.calcRows(0, M-1, 0, N-1,
-                state_directions, T, scores, rows, track, **kw)
-        end_state_only = numpy.array([(len(T)-1, 0, 1, 1)])
-        result = pair.calcRows(M-1, M, N-1, N,
-                end_state_only, T, scores, rows, track, **kw)
-        return result
-    
-    def _calc_local(self, pair, scores, kw, state_directions, T, rows, track):
-        (M, N) = pair.size
-        result =  pair.calcRows(1, M-1, 1, N-1,
-                state_directions, T, scores, rows, track, **kw)
-        return result
-    
+        
     def __getitem__(self, index):
         assert len(index) == 2, index
         return PairEmissionProbs(self.pair[index], self.bins)
-    
+                
     def hirschberg(self, TM, dp_options):
         """linear-space alignment algorithm
         A linear space algorithm for computing maximal common subsequences.
@@ -566,64 +630,91 @@ class PairEmissionProbs(object):
         Dan Hirshberg
         """
         (states, T) = TM
-        row_count = self.pair.size[0] - 2
-        half = row_count // 2
         
+        # This implementation is slightly complicated by the need to handle 
+        # alignments of alignments, because a subalignment may have an indel
+        # spanning the midpoint where we want to divide the problem in half.
+        # That must be the sense in which the fatter and slower method used
+        # in "Prank" (Loytynoja A, Goldman N. 2005) is "computationally more 
+        # attractive": for them there is only one link in the list:
+        links = self.pair.children[0].midlinks()
+
         T2 = T.copy()
         T2[1:-1:,-1] = 1.0  # don't count the end state transition twice
-        (prows1, last_row1) = self[:half, :].dp(
-                (states, T2), dp_options, last_row=True)
+        last_row1 = self.scores_at_rows(
+                (states, T2), dp_options, last_row=[i for (i,j) in links])
         
         T2 = T.copy()
         T2[0, 1:-1] = 1.0  # don't count the begin state transition twice
-        (prows2, last_row2) = self[half:, :].dp(
-                (states, T2), dp_options, last_row=True, backward=True)
-        
-        middle_row = (last_row1 + last_row2[:,::-1,:])
-        anchor = numpy.argmax(middle_row.flat)
-        (link, rest) = divmod(anchor, middle_row.shape[1]*middle_row.shape[2])
-        (anchor, anchor_state) = divmod(rest, middle_row.shape[2])
-        join1 = prows1[link]
-        join2 = row_count - prows2[link]
-        #if join1 != join2:
-        #    print half, link, prows1, prows2, self.pair.size
-        #    print join1, join2
+        last_row2 = self.scores_at_rows(
+                (states, T2), dp_options, 
+                last_row=[j for (i,j) in links], 
+                backward=True)
+        middle_row = (last_row1 + last_row2)
+        (link, anchor, anchor_state) = numpy.unravel_index(
+            numpy.argmax(middle_row.flat), middle_row.shape)
         score = middle_row[link, anchor, anchor_state]
-        #print 'h', self.pair.size, [half, anchor], anchor_state, score
+        (join1, join2) = links[link]
         T2 = T.copy()
         T2[-1] = 0.0
         T2[anchor_state, -1] = 1.0  # Must end with the anchor's state
-        (s1, (tb1a, tb2a)) = self[:join1, :anchor].dp((states, T2), dp_options)
+        (s1, tb_a) = self[:join1, :anchor].dp((states, T2), dp_options)
         T2 = T.copy()
         T2[0, :] = T[anchor_state, :]  # Starting from the anchor's state
-        (s2, (tb1b, tb2b)) = self[join2:, anchor:].dp((states, T2), dp_options)
-        tb1b = [(state, (x+half, y+anchor), dxy)
-                for (state, (x,y), dxy) in tb1b]
-        tb = tb1a+tb1b
-        tb2 = self._nice_tb(states, tb)
-        #print 'scores', s1+s2, score   # XXX should be equal
-        # Same return tuple struct as for self.dp(..., tb=...)
-        return score, (tb, tb2)
+        (s2, tb_b) = self[join2:, anchor:].dp((states, T2), dp_options)
+        tb = tb_a + tb_b.offset(join2, anchor)
+        # Same return as for self.dp(..., tb=...)
+        return score, tb
     
-    def dp(self, TM, dp_options, tb=None, last_row=False, backward=False):
-        # tb: viterbi traceback
-        # last_row: for hirschberg
+    def scores_at_rows(self, TM, dp_options, last_row, backward=False):
+        """A score array shaped [rows, columns, states] but only for those
+        row numbers requested.  Used by Hirschberg algorithm"""
+        (M, N) = self.pair.size
         (state_directions, T) = TM
+        reverse = bool(dp_options.backward) ^ bool(backward)
+        cells = []
+        p_rows = sorted(set(last_row))
+        if reverse:
+            p_rows.reverse()
+        for i in p_rows:
+            for j in range(0, N-1):
+                for state in range(len(T)-1):
+                    if reverse:
+                        cells.append((state, (M-2-i, N-2-j)))
+                    else:
+                        cells.append((state, (i, j)))
+        probs = self.dp(TM, dp_options, cells=cells, backward=backward)
+        probs = numpy.array(probs)
+        probs.shape = (len(p_rows), N-1, len(T)-1)
+        result = numpy.array([
+            probs[p_rows.index(i)] for i in last_row])
+        return result
         
-        if dp_options.viterbi and not last_row:
-            problem_size = numpy.product(self.pair.size) * len(T)
+    def dp(self, TM, dp_options, cells=None, backward=False):
+        """Score etc. from a Dynamic Programming function applied to this pair.
+        
+        TM - (state_directions, array) describing the Transition Matrix.
+        dp_options - instance of DPFlags indicating algorithm etc.
+        cells - List of (state, posn) for which posterior probs are requested.
+        backward - run algorithm in reverse order.
+        """
+        (state_directions, T) = TM
+        if dp_options.viterbi and cells is None:
+            encoder = self.pair.getPointerEncoding(len(T))
+            problem_dimensions = self.pair.size + [len(T)]
+            problem_size = numpy.product(problem_dimensions)
             if problem_size > HIRSCHBERG_LIMIT and self.pair.size[0]-2 >= 3:
-                memory = 4*problem_size/10**6
+                memory = problem_size * encoder.bytes / 10**6
                 if dp_options.local:
                     LOG.warn('Local alignment will use > %sMb.' % memory)
-                elif tb is not None:
+                elif cells is not None:
                     LOG.warn('Posterior probs will use > %sMb.' % memory)
                 else:
                     assert not backward
                     return self.hirschberg(TM, dp_options)
-            track = self.pair.getEmptyStateArray(len(T))
+            track = encoder.getEmptyArray(problem_dimensions)
         else:
-            track = None
+            track = encoder = None
         
         kw = dict(
                 use_scaling=dp_options.use_scaling,
@@ -637,77 +728,54 @@ class PairEmissionProbs(object):
         if backward:
             pair = self.pair.backward()
             origT = T
-            T = numpy.zeros(T.shape, Float)
+            T = numpy.zeros(T.shape, float)
             T[1:-1,1:-1] = numpy.transpose(origT[1:-1,1:-1])
             T[0,:] = origT[:, -1]
             T[:,-1] = origT[0,:]
         else:
             pair = self.pair
+        
         if dp_options.use_logs:
             T = numpy.log(T)
         
-        scores = self._getEmissionProbs(dp_options.use_logs,
-                dp_options.use_cost_function)
+        scores = self._getEmissionProbs(
+                dp_options.use_logs, dp_options.use_cost_function)
         
         rows = pair.getEmptyScoreArrays(len(T), dp_options)
-        if last_row:
-            (M, N) = pair.size
-            #print 'row', M-2, 'of', M, backward
-            tb = []
-            p_rows = self.pair.penultimateRows()
-            for i in p_rows:
-                for j in range(0, N-1):
-                    for state in range(len(T)-1):
-                        tb.append((state, (M-2, j), None))
+        
+        if cells is not None:
             assert not dp_options.local
             result = self._calc_global_probs(
-                    pair, scores, kw, state_directions, T, rows, tb,
+                    pair, scores, kw, state_directions, T, rows, cells,
                     backward)
-            result = numpy.array(result)
-            result.shape = (len(p_rows), N-1, len(T)-1)
-            result = (p_rows, result)
-        elif tb is not None:
-            assert not dp_options.local
-            result = self._calc_global_probs(
-                    pair, scores, kw, state_directions, T, rows, tb,
-                    backward)
-            if last_row:
-                result = numpy.array(result)
-                result.shape = (N-1, len(T)-1)
         else:
-            calc = [self._calc_global, self._calc_local][dp_options.local]
-            (maxpos, state, score) = calc(
-                    pair, scores, kw, state_directions, T, rows, track)
-            
+            (M, N) = pair.size
+            if dp_options.local:
+                (maxpos, state, score) =  pair.calcRows(1, M-1, 1, N-1,
+                    state_directions, T, scores, rows, track, encoder, **kw)
+            else:
+                pair.calcRows(0, M-1, 0, N-1,
+                    state_directions, T, scores, rows, track, encoder, **kw)
+                end_state_only = numpy.array([(len(T)-1, 0, 1, 1)])
+                (maxpos, state, score) = pair.calcRows(M-1, M, N-1, N,
+                    end_state_only, T, scores, rows, track, encoder, **kw)
+                    
             if track is None:
                 result = score
             else:
-                tb = self.pair.traceback(track, maxpos, state,
+                tb = self.pair.traceback(track, encoder, maxpos, state,
                     skip_last = not dp_options.local)
-                tb2 = self._nice_tb(state_directions, tb)
-                result = (score, (tb, tb2))
+                result = (score, tb)
         return result
-    
-    def _nice_tb(self, state_directions, tb):
-        bin_map = dict((state, bin) for (state, bin, dx, dy) in
-            state_directions)
-        result = []
-        for (state, posn, (dx, dy)) in tb:
-            pos = [[None, i-1][not not d] for (i,d) in zip(
-                    posn, [dx, dy])]
-            result.append((bin_map.get(int(state), None), pos))
-        return result
-    
-    def getAlignable(self, aligned_positions, use_cost_function=False,
-            ratio=None):
+        
+    def getAlignable(self, aligned_positions, ratio=None):
         assert ratio is None, "already 2-branched"
-        assert not use_cost_function
         children = self.pair.children # alignables
         leaves = [c.leaf for c in children]
         aligned_positions = [posn for (bin, posn) in aligned_positions]
         pog = self.pair.getPOG(aligned_positions)
         edge = LikelihoodTreeEdge(leaves, 'parent', pog.getAlignedPositions())
-        (plhs, gap_scores) = self.makePartialLikelihoods(use_cost_function)
+        (plhs, gapscores) = self.makePartialLikelihoods(use_cost_function=False)
         plh = self.pair.edge2plh(edge, plhs)
         assert len(plh) == 1, ('bins!', len(plh))
         leaf = edge.asLeaf(plh[0]) # like profile
@@ -744,9 +812,6 @@ class ReversiblePairEmissionProbs(object):
     
     def __init__(self, pair, bins, length):
         self.pair = pair
-        #self.mprobs = mprobs
-        #self.Qds = Qds
-        #self.rates = rates
         self.bins = bins
         self.length = length
         self.midpoint = self._makePairEmissionProbs(0.5)
@@ -760,25 +825,27 @@ class ReversiblePairEmissionProbs(object):
         pbins = [bin.forLengths(*lengths) for bin in self.bins]
         return PairEmissionProbs(self.pair, pbins)
     
-    def getAlignable(self, a_p, ratio=None, use_cost_function=False):
+    def getAlignable(self, a_p, ratio=None):
         # a_p alignment positions
         if ratio in [None, 0.5]:
             ep = self.midpoint
         else:
             ep = self._makePairEmissionProbs(ratio=ratio)
-        return ep.getAlignable(a_p, use_cost_function)
+        return ep.getAlignable(a_p)
     
     def makePairHMM(self, transition_matrix):
         return PairHMM(self, transition_matrix)
     
 
 class DPFlags(object):
-    def __init__(self, viterbi, local=False, use_logs=False,
-            use_cost_function=None, use_scaling=None, backward=False):
+    def __init__(self, viterbi, local=False, use_logs=None,
+            use_cost_function=True, use_scaling=None, backward=False):
+        if use_logs is None:
+            use_logs = viterbi and not use_scaling
         if use_scaling is None:
             use_scaling = not use_logs
-        if use_cost_function is None:
-            use_cost_function = True
+        if use_logs:
+            assert viterbi and not use_scaling
         self.use_cost_function = use_cost_function
         self.local = local
         self.use_logs = use_logs
@@ -814,19 +881,19 @@ class PairHMM(object):
         return self._getDPResult(viterbi=False, **kw)
     
     def _getPosteriorProbs(self, tb, **kw):
+        cells = tb.asStatePosTuples()
         score = self.getForwardScore(**kw)
         dp_options = DPFlags(viterbi=False, **kw)
-        fwd = self.emission_probs.dp(self._transition_matrix, dp_options, tb)
+        fwd = self.emission_probs.dp(self._transition_matrix, dp_options, cells)
         (N, M) = self.emission_probs.pair.size
-        tb = [(state, (N-x-2, M-y-2), (dx, dy))
-                for (state, (x,y), (dx, dy)) in tb]
+        cells = [(state, (N-x-2, M-y-2)) for (state, (x,y)) in cells]
         tb.reverse()
-        back = self.emission_probs.dp(self._transition_matrix, dp_options, tb,
-            backward=True)[::-1]
-        return fwd + back - score
+        bck = self.emission_probs.dp(self._transition_matrix, dp_options, cells,
+                backward=True)[::-1]
+        return fwd + bck - score
     
     def getViterbiPath(self, **kw):
-        result = self._getDPResult(viterbi=True, **kw)
+        result = self._getDPResult(viterbi=True,**kw)
         return ViterbiPath(self, result, **kw)
     
     def getViterbiScoreAndAlignment(self, ratio=None, **kw):
@@ -837,8 +904,9 @@ class PairHMM(object):
     def getLocalViterbiScoreAndAlignment(self, posterior_probs=False, **kw):
         # Only for pairwise.  Merge with getViterbiScoreAndAlignable above.
         # Local and POGs doesn't mix well.
-        (vscore, (tb, aligned_positions)) = self._getDPResult(
-                viterbi=True, local=True, **kw)
+        (vscore, tb) = self._getDPResult(viterbi=True, local=True, **kw)
+        (state_directions, T) = self._transition_matrix
+        aligned_positions = tb.asBinPosTuples(state_directions)
         seqs = self.emission_probs.pair.getSeqNamePairs()
         aligned_positions = [posn for (bin, posn) in aligned_positions]
         word_length = self.emission_probs.pair.alphabet.getMotifLen()
@@ -852,7 +920,9 @@ class PairHMM(object):
 
 class ViterbiPath(object):
     def __init__(self, pair_hmm, result, **kw):
-        (self.vscore, (self.tb, self.aligned_positions)) = result
+        (self.vscore, self.tb) = result
+        (state_directions, T) = pair_hmm._transition_matrix
+        self.aligned_positions = self.tb.asBinPosTuples(state_directions)
         self.pair_hmm = pair_hmm
         self.kw = kw
     
