@@ -26,6 +26,7 @@ __all__ = ['blast',
            'pfold',
            'pknotsrg',
            'raxml',
+           'rdp_classifier',
            'rnaalifold',
            'rnaforester',
            'rnashapes',
@@ -41,7 +42,7 @@ __credits__ = ["Jeremy Widmann", "Catherine Lozuopone", "Gavin Huttley",
                     "Rob Knight", "Zongzhi Liu", "Sandra Smit", "Micah Hamady",
                     "Greg Caporaso", "Mike Robeson", "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
