@@ -5,18 +5,19 @@ from string import strip
 from os import remove, access, F_OK, environ, path
 from cogent.app.parameters import FlagParameter, ValuedParameter, MixedParameter
 from cogent.app.util import CommandLineApplication, ResultPath, \
-    get_tmp_filename, guess_input_handler
+    get_tmp_filename, guess_input_handler, ApplicationNotFoundError
 from cogent.parse.fasta import FastaFinder, LabeledRecordFinder, is_fasta_label
 from cogent.parse.blast import LastProteinIds9, QMEBlast9, QMEPsiBlast9, BlastResult
+from cogent.util.misc import app_path
 from random import choice
 from copy import copy
 
 __author__ = "Micah Hamady"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Zongzhi Liu", "Micah Hamady", "Jeremy Widmann",
-                    "Catherine Lozupone", "Rob Knight"]
+                    "Catherine Lozupone", "Rob Knight","Greg Caporaso"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Micah Hamady"
 __email__ = "hamady@colorado.edu"
 __status__ = "Prototype"
@@ -134,15 +135,17 @@ class Blast(CommandLineApplication):
 
     }
 
+    _executable = 'blastall'
+
     _parameters = {}
     _parameters.update(_common_options)
 
     def __init__(self, cur_options, command, blast_mat_root=None,
                  extra_env="",
                  params=None,InputHandler=None,
-                 SuppressStderr=None, SuppressStdout=None,WorkingDir=None):
+                 SuppressStderr=None, SuppressStdout=None,WorkingDir=None,\
+                 HALT_EXEC=False):
         """ Initialize blast """
-
         # update options
         self._parameters.update(cur_options)
 
@@ -151,7 +154,14 @@ class Blast(CommandLineApplication):
             self._command = "export BLASTMAT=%s;%s%s" % (blast_mat_root, 
                                                     extra_env, command)
         else:
+            # Determine if blast is installed and raise an ApplicationError 
+            # if not -- this is done here so the user will get the most 
+            # informative error message available.  
+            self._error_on_missing_application(params)
+                 
+            # Otherwise raise error about $BLASTMAT not being set
             if not ('BLASTMAT' in environ or access(path.expanduser("~/.ncbirc"), F_OK) or access(".ncbirc", F_OK)) :
+                ## SHOULD THIS BE CHANGED TO RAISE AN ApplicationError?
                 raise RuntimeError, \
         """BLAST cannot run if the BLASTMAT environment variable is not set.
 
@@ -188,7 +198,15 @@ call the Standalone BLAST program from or in your root directory.
 
         super(Blast, self).__init__(params=params,
                     InputHandler=InputHandler,SuppressStderr=SuppressStderr,
-                    SuppressStdout=SuppressStdout,WorkingDir=WorkingDir)
+                    SuppressStdout=SuppressStdout,WorkingDir=WorkingDir,\
+                    HALT_EXEC=HALT_EXEC)
+
+    def _error_on_missing_application(self,params):
+        """ Raise an ApplicationNotFoundError if the app is not accessible
+        """
+        if not app_path('blastall'):
+            raise ApplicationNotFoundError,\
+             "Cannot find blastall. Is it installed? Is it in your path?"
 
     def _input_as_seqs(self,data):
         lines = []
@@ -197,6 +215,14 @@ call the Standalone BLAST program from or in your root directory.
             lines.append(''.join(['>',str(i+1)]))
             lines.append(s)
         return self._input_as_lines(lines)
+        
+    def _input_as_seq_id_seq_pairs(self,data):
+        lines = []
+        for seq_id,seq in data:
+            #will number the sequences 1,2,3,etc.
+            lines.append(''.join(['>',str(seq_id)]))
+            lines.append(seq)
+        return self._input_as_lines(lines)
 
     def _input_as_lines(self,data):
         if data:
@@ -302,7 +328,8 @@ class PsiBlast(Blast):
     def __init__(self, blast_mat_root=None, params=None,
                  extra_env="",
                  InputHandler=None,SuppressStderr=None,
-                 SuppressStdout=None,WorkingDir=None):
+                 SuppressStdout=None,WorkingDir=None,
+                 HALT_EXEC=False):
         """ Initialize the Psi-Blast"""
         super(PsiBlast, self).__init__(self._options,
                     "blastpgp",
@@ -310,7 +337,8 @@ class PsiBlast(Blast):
                     blast_mat_root=blast_mat_root,
                     params=params,
                     InputHandler=InputHandler,SuppressStderr=SuppressStderr,
-                    SuppressStdout=SuppressStdout,WorkingDir=WorkingDir)
+                    SuppressStdout=SuppressStdout,WorkingDir=WorkingDir,
+                    HALT_EXEC=HALT_EXEC)
 
 
 # should probably go into blastall superclass. it's late, works for now
@@ -362,7 +390,8 @@ class Blastall(Blast):
     def __init__(self, blast_mat_root=None, params=None,
                  extra_env="",
                  InputHandler=None,SuppressStderr=None,
-                 SuppressStdout=None,WorkingDir=None):
+                 SuppressStdout=None,WorkingDir=None,
+                 HALT_EXEC=False):
         """ Initialize the blastall"""
         super(Blastall, self).__init__(BLASTALL_OPTIONS,
                     "blastall", 
@@ -370,7 +399,8 @@ class Blastall(Blast):
                     extra_env=extra_env,
                     params=params,
                     InputHandler=InputHandler,SuppressStderr=SuppressStderr,
-                    SuppressStdout=SuppressStdout,WorkingDir=WorkingDir)
+                    SuppressStdout=SuppressStdout,WorkingDir=WorkingDir,
+                    HALT_EXEC=HALT_EXEC)
 class MpiBlast(Blast):
     """mpblast application controller - Prototype """
 
@@ -450,7 +480,8 @@ class MpiBlast(Blast):
                  config_file="/quicksand2/downloads2/mpiblast/mpiblast.conf",
                  num_db_frags=40,
                  InputHandler=None,SuppressStderr=None,
-                 SuppressStdout=None,WorkingDir=None):
+                 SuppressStdout=None,WorkingDir=None,
+                 HALT_EXEC=False):
         """ Initialize mpiblast"""
         if config_file:
             params["--config-file"] = config_file
@@ -462,7 +493,8 @@ class MpiBlast(Blast):
                         shared_root),
                     params=params,
                     InputHandler=InputHandler,SuppressStderr=SuppressStderr,
-                    SuppressStdout=SuppressStdout,WorkingDir=WorkingDir)
+                    SuppressStdout=SuppressStdout,WorkingDir=WorkingDir,
+                    HALT_EXEC=HALT_EXEC)
 
 class FastaCmd(CommandLineApplication):
     """FastaCmd application controller - Prototype"""
@@ -591,7 +623,8 @@ def blast_seqs(seqs,
                  WorkingDir=None,
                  SuppressStderr=None,
                  SuppressStdout=None,
-                 input_handler=None
+                 input_handler=None,
+                 HALT_EXEC=False
                  ):
     """Blast list of sequences.
 
@@ -629,7 +662,8 @@ def blast_seqs(seqs,
                    InputHandler=ih,
                    WorkingDir=WorkingDir,
                    SuppressStderr=SuppressStderr,
-                   SuppressStdout=SuppressStdout)
+                   SuppressStdout=SuppressStdout,
+                   HALT_EXEC=HALT_EXEC)
 
     return blast_app(seqs)
 
