@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 """
 Provides an application controller for the commandline version of:
-Infernal 1.0rc1
+Infernal 1.0
 """
 from cogent.app.parameters import FlagParameter, ValuedParameter, FilePath
 from cogent.app.util import CommandLineApplication, ResultPath, get_tmp_filename
 from cogent.parse.fasta import MinimalFastaParser
-from cogent.parse.rfam import MinimalRfamParser,RfamParser
+from cogent.parse.rfam import MinimalRfamParser, ChangedSequence, \
+    ChangedRnaSequence, ChangedDnaSequence
 from cogent.parse.infernal import CmsearchParser
 from cogent.core.moltype import DNA, RNA
 from cogent.core.alignment import SequenceCollection, Alignment, DataError
@@ -18,15 +19,23 @@ __author__ = "Jeremy Widmann"
 __copyright__ = "Copyright 2007-2009 2007, The Cogent Project"
 __credits__ = ["Jeremy Widmann"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Jeremy Widmann"
 __email__ = "jeremy.widmann@colorado.edu"
 __status__ = "Development"
 
 MOLTYPE_MAP = {'DNA':'--dna',\
+                DNA:'--dna',\
                'RNA':'--rna',\
+                RNA:'--rna',\
                }
                
+SEQ_CONSTRUCTOR_MAP = {'DNA':ChangedDnaSequence,\
+                        DNA:ChangedDnaSequence,\
+                       'RNA':ChangedRnaSequence,\
+                        RNA:ChangedRnaSequence,\
+                       }
+
 class Cmalign(CommandLineApplication):
     """cmalign application controller."""
     _options = {
@@ -1267,7 +1276,6 @@ def cmbuild_from_alignment(aln, structure_string, refine=False, \
     
     cm_file = res['CmFile'].read()
     
-    
     if return_alignment:
         #If alignment was refined, return refined alignment and structure,
         # otherwise return original alignment and structure.
@@ -1298,7 +1306,8 @@ def cmbuild_from_file(stockholm_file_path, refine=False,return_alignment=False,\
     """
     #get alignment and structure string from stockholm file.
     info, aln, structure_string = \
-        list(MinimalRfamParser(open(stockholm_file_path,'U')))[0]
+        list(MinimalRfamParser(open(stockholm_file_path,'U'),\
+            seq_constructor=ChangedSequence))[0]
     
     #call cmbuild_from_alignment.
     res = cmbuild_from_alignment(aln, structure_string, refine=refine, \
@@ -1306,7 +1315,8 @@ def cmbuild_from_file(stockholm_file_path, refine=False,return_alignment=False,\
     return res
 
 def cmalign_from_alignment(aln, structure_string, seqs, moltype,\
-    include_aln=True,refine=False, return_stdout=False,params=None):
+    include_aln=True,refine=False, return_stdout=False,params=None,\
+    cmbuild_params=None):
     """Uses cmbuild to build a CM file, then cmalign to build an alignment.
     
         - aln: an Alignment object or something that can be used to construct
@@ -1334,37 +1344,51 @@ def cmalign_from_alignment(aln, structure_string, seqs, moltype,\
     int_map = SequenceCollection(int_map,MolType=moltype)
     
     cm_file, aln_file_string = cmbuild_from_alignment(aln, structure_string,\
-        refine=refine,return_alignment=True)
+        refine=refine,return_alignment=True,params=cmbuild_params)
+    
+    if params is None:
+        params = {}    
+    params.update({MOLTYPE_MAP[moltype]:True})
     
     app = Cmalign(InputHandler='_input_as_paths',WorkingDir='/tmp',\
         params=params)
     app.Parameters['--informat'].on('FASTA')
     
+    #files to remove that aren't cleaned up by ResultPath object
+    to_remove = []    
     #turn on --withali flag if True.
     if include_aln:
         app.Parameters['--withali'].on(\
             app._tempfile_as_multiline_string(aln_file_string))
+        #remove this file at end
+        to_remove.append(app.Parameters['--withali'].Value)
     
     seqs_path = app._input_as_multiline_string(int_map.toFasta())
     cm_path = app._tempfile_as_multiline_string(cm_file)
-    paths = [cm_path,seqs_path]
     
+    #add cm_path to to_remove
+    to_remove.append(cm_path)
+    paths = [cm_path,seqs_path]
+
     app.Parameters['-o'].on(get_tmp_filename(app.WorkingDir))
+    
     res = app(paths)
     
     info, aligned, struct_string = \
-        list(MinimalRfamParser(res['Alignment'].readlines()))[0]
-    
+        list(MinimalRfamParser(res['Alignment'].readlines(),\
+            seq_constructor=SEQ_CONSTRUCTOR_MAP[moltype]))[0]
     
     #Make new dict mapping original IDs
     new_alignment={}
-    for k,v in aligned.items():
+    for k,v in aligned.NamedSeqs.items():
         new_alignment[int_keys.get(k,k)]=v
     #Create an Alignment object from alignment dict
     new_alignment = Alignment(new_alignment,MolType=moltype)
     
     std_out = res['StdOut'].read()
+    #clean up files
     res.cleanUp()
+    for f in to_remove: remove(f)
     
     if return_stdout:
         return new_alignment, struct_string, std_out
@@ -1401,10 +1425,14 @@ def cmalign_from_file(cm_file_path, seqs, moltype, alignment_file_path=None,\
     #Create SequenceCollection from int_map.
     int_map = SequenceCollection(int_map,MolType=moltype)
     
+    if params is None:
+        params = {}
+    params.update({MOLTYPE_MAP[moltype]:True})
+    
     app = Cmalign(InputHandler='_input_as_paths',WorkingDir='/tmp',\
         params=params)
     app.Parameters['--informat'].on('FASTA')
-    
+        
     #turn on --withali flag if True.
     if include_aln:
         if alignment_file_path is None:
@@ -1419,7 +1447,8 @@ def cmalign_from_file(cm_file_path, seqs, moltype, alignment_file_path=None,\
     res = app(paths)
     
     info, aligned, struct_string = \
-        list(MinimalRfamParser(res['Alignment'].readlines()))[0]
+        list(MinimalRfamParser(res['Alignment'].readlines(),\
+            seq_constructor=SEQ_CONSTRUCTOR_MAP[moltype]))[0]
     
     
     #Make new dict mapping original IDs
@@ -1470,9 +1499,12 @@ def cmsearch_from_alignment(aln, structure_string, seqs, moltype, cutoff=0.0,\
     app.Parameters['--informat'].on('FASTA')
     app.Parameters['-T'].on(cutoff)
     
+    to_remove = []
+    
     seqs_path = app._input_as_multiline_string(int_map.toFasta())
     cm_path = app._tempfile_as_multiline_string(cm_file)
     paths = [cm_path,seqs_path]
+    to_remove.append(cm_path)
     
     app.Parameters['--tabfile'].on(get_tmp_filename(app.WorkingDir))
     res = app(paths)
@@ -1485,7 +1517,8 @@ def cmsearch_from_alignment(aln, structure_string, seqs, moltype, cutoff=0.0,\
             search_results[i][0]=int_keys.get(label,label)
     
     res.cleanUp()
-
+    for f in to_remove:remove(f)
+    
     return search_results
 
 def cmsearch_from_file(cm_file_path, seqs, moltype, cutoff=0.0, params=None):
