@@ -2,20 +2,21 @@
 import commands
 from sys import platform
 from os import remove,system,mkdir,getcwd,close,sep
-from os.path import isabs
+from random import choice
+from os.path import isabs, exists
+from numpy import zeros, array, nonzero, max
 from cogent.app.parameters import Parameter, FlagParameter, ValuedParameter,\
     MixedParameter,Parameters, _find_synonym, is_not_None, FilePath
 from cogent.util.misc import if_
 from cogent.util.transform import cartesian_product
-from random import choice
-from numpy import zeros, array, nonzero, max
+from cogent.util.misc import app_path
 
 __author__ = "Sandra Smit and Greg Caporaso"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Greg Caporaso", "Sandra Smit", "Micah Hamady",
                     "Jeremy Widmann", "Rob Knight", "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Sandra Smit"
 __email__ = "sandra.smit@colorado.edu"
 __status__ = "Development"
@@ -28,6 +29,9 @@ _all_chars = _chars + _chars.upper() + "0123456790"
 class ApplicationError(OSError):
     pass
    
+class ApplicationNotFoundError(ApplicationError):
+    pass
+   
 class ResultPath(object):
     """ Hold a file path a boolean value specifying whether file was written
     """
@@ -158,6 +162,9 @@ class CommandLineApplication(Application):
             HALT_EXEC: if True, raises exception w/ command output just
             before execution, doesn't clean up temp files. Default False.
         """
+        # Determine if the application is installed, and raise an error if not
+        self._error_on_missing_application(params)
+        
         # set attributes to parameter that was passed in or class default
         if InputHandler is not None:
             self.InputHandler = InputHandler
@@ -234,13 +241,21 @@ class CommandLineApplication(Application):
         # We only want to keep the exit status so do a right bitwise shift to 
         # get rid of the signal number byte
         exit_status = system(command) >> 8
-      
+        
         # Determine if error should be raised due to exit status of 
         # appliciation
         if not self._accept_exit_status(exit_status):
             raise ApplicationError, \
              'Unacceptable application exit status: %s, command: %s'\
                 % (str(exit_status),command)
+        # bash returns 127 as the exit status if the command could not
+        # be found -- raise an ApplicationError on status == 127.
+        # elif exit_status == 127:
+        #     raise ApplicationError, \
+        #      "Could not execute %s. Is it installed? Is it in your path?"\
+        #      % self._command
+        # else:
+        #     pass
         
         # open the stdout and stderr if not being suppressed
         out = None
@@ -380,6 +395,29 @@ class CommandLineApplication(Application):
             pass 
 
     WorkingDir = property(_get_WorkingDir,_set_WorkingDir)
+    
+    def _error_on_missing_application(self,params):
+        """ Raise an ApplicationNotFoundError if the app is not accessible
+        
+            This method checks in the system path (usually $PATH) or for
+            the existence of self._command. If self._command is not found
+            in either place, an ApplicationNotFoundError is raised to
+            inform the user that the application they are trying to access is
+            not available.
+            
+            This method should be overwritten when self._command does not
+            represent the relevant executable (e.g., self._command = 'prog -a')
+            or in more complex cases where the file to be executed may be 
+            passed as a parameter (e.g., with java jar files, where the 
+            jar file is passed to java via '-jar'). It can also be overwritten
+            to by-pass testing for application presence by never raising an 
+            error.
+        """
+        command = self._command
+        if not (exists(command) or app_path(command)):
+            raise ApplicationNotFoundError,\
+             "Cannot find %s. Is it installed? Is it in your path?"\
+             % command
 
     
     def _accept_exit_status(self,exit_status):
@@ -413,7 +451,7 @@ class CommandLineApplication(Application):
 
 
     def getTmpFilename(self, tmp_dir="/tmp",prefix='tmp',suffix='.txt',\
-        include_class_id=False):
+        include_class_id=False,result_constructor=FilePath):
         """ Return a temp filename
 
             tmp_dir: path for temp file
@@ -426,6 +464,11 @@ class CommandLineApplication(Application):
              testing: if temp files are being left behind by tests, you can
              turn this on in here (temporarily) to find out which tests are
              leaving the temp files.
+            result_constructor: the constructor used to build the result
+             (default: cogent.app.parameters.FilePath). Note that joining 
+             FilePath objects with one another or with strings, you must use
+             the + operator. If this causes trouble, you can pass str as the 
+             the result_constructor.
         """
 
         # check not none
@@ -451,10 +494,10 @@ class CommandLineApplication(Application):
             # Directory already exists
             pass
         # note: it is OK to join FilePath objects with + 
-        return FilePath(tmp_dir) + FilePath(prefix) + \
-            FilePath(''.join([choice(_all_chars) \
+        return result_constructor(tmp_dir) + result_constructor(prefix) + \
+            result_constructor(''.join([choice(_all_chars) \
              for i in range(self.TmpNameLen)])) +\
-            FilePath(suffix)
+            result_constructor(suffix)
 
 class ParameterIterBase:
     """Base class for parameter iteration objects
@@ -639,9 +682,22 @@ def cmdline_generator(param_iter, PathToBin=None, PathToCmd=None, \
         
             yield ' '.join(cmdline)
        
-def get_tmp_filename(tmp_dir="/tmp", prefix="tmp", suffix=".txt"):
-    """
-    Generate temp filename
+def get_tmp_filename(tmp_dir="/tmp", prefix="tmp", suffix=".txt",\
+    result_constructor=FilePath):
+    """ Generate a temporary filename and return as a FilePath object
+    
+        tmp_dir: the directory to house the tmp_filename (default: '/tmp')
+        prefix: string to append to beginning of filename (default: 'tmp')
+            Note: It is very useful to have prefix be descriptive of the
+            process which is creating the temporary file. For example, if 
+            your temp file will be used to build a temporary blast database, 
+            you might pass prefix=TempBlastDB
+        suffix: the suffix to be appended to the temp filename (default '.txt')
+        result_constructor: the constructor used to build the result filename
+            (default: cogent.app.parameters.FilePath). Note that joining 
+            FilePath objects with one another or with strings, you must use
+            the + operator. If this causes trouble, you can pass str as the 
+            the result_constructor.
     """
     # check not none
     if not tmp_dir:
@@ -652,23 +708,26 @@ def get_tmp_filename(tmp_dir="/tmp", prefix="tmp", suffix=".txt"):
 
     chars = "abcdefghigklmnopqrstuvwxyz"
     picks = chars + chars.upper() + "0123456790"
-    return FilePath(tmp_dir) + FilePath(prefix) +\
-        FilePath("%s%s" % \
+    return result_constructor(tmp_dir) + result_constructor(prefix) +\
+        result_constructor("%s%s" % \
         (''.join([choice(picks) for i in range(20)]),suffix))
 
 def guess_input_handler(seqs, add_seq_names=False):
     """Returns the name of the input handler for seqs."""
     if isinstance(seqs, str):
         if '\n' in seqs:    #can't be a filename...
-            ih = '_input_as_multiline_string'
+            return '_input_as_multiline_string'
         else:               #assume it was a filename
-            ih = '_input_as_string'
+            return '_input_as_string'
             #Uncommenting the next line causes errors in muscle tests - Micah?
-            #ih = '_input_as_path'
-    elif add_seq_names:
-        ih = '_input_as_seqs'
-    else:
-        ih = '_input_as_lines'
-    return ih
+            # return '_input_as_path'
+    
+    if isinstance(seqs,list) and len(seqs) and isinstance(seqs[0],tuple):
+        return '_input_as_seq_id_seq_pairs'
+    
+    if add_seq_names:
+        return '_input_as_seqs'
+    
+    return '_input_as_lines'
 
 
