@@ -2,14 +2,14 @@
 """cluster: provides tools for clustering
 """
 
-__all__ = ['metric_scaling', 'UPGMA', 'nmds']
+__all__ = ['goodness_of_fit', 'metric_scaling', 'procrustes', 'UPGMA', 'nmds']
 
 __author__ = ""
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Catherine Lozuopone", "Rob Knight", "Peter Maxwell",
     "Justin Kuczynski"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Catherine Lozupone"
 __email__ = "lozupone@colorado.edu"
 __status__ = "Production"
