@@ -35,7 +35,7 @@ __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight",
                     "Andrew Butterfield"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -595,7 +595,8 @@ class Alphabet(Enumeration):
         
         Always returns the same object.
         """
-        assert not self.includesGapMotif()
+        if self.includesGapMotif():
+            return self
         if not hasattr(self, 'Gapped'):
             self.Gapped = self._with(list(self) + [self.getGapMotif()])
         return self.Gapped
@@ -654,6 +655,24 @@ class Alphabet(Enumeration):
         
         return tuple(motif_set)
     
+    def adaptMotifProbs(self, motif_probs):
+        """Prepare an array or dictionary of probabilities for use with
+        this alphabet by checking size and order"""
+        if hasattr(motif_probs, 'keys'):
+            sample = motif_probs.keys()[0]
+            if sample not in self:
+                raise ValueError("Can't find motif %s in alphabet" %
+                                sample)
+            motif_probs = numpy.array(
+                    [motif_probs[motif] for motif in self])
+        else:
+            if len(motif_probs) != len(self):
+                if len(motif_probs) != len(self):
+                    raise ValueError("Can't match %s probs to %s alphabet" %
+                            (len(motif_probs), len(self)))
+            motif_probs = numpy.asarray(motif_probs)
+        assert abs(sum(motif_probs)-1.0) < 0.0001, motif_probs
+        return motif_probs
 
 class CharAlphabet(Alphabet):
     """Holds an alphabet whose items are single chars.
