@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 
-__all__ = ['energy_params', 'molecular_weight']
+__all__ = ['energy_params', 'molecular_weight', 'protein_properties', \
+           'ligand_properties', 'nucleic_properties']
 
 __author__ = ""
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
-__credits__ = ["Rob Knight", "Amanda Birmingham"]
+__credits__ = ["Rob Knight", "Amanda Birmingham", "Marcin Cieslik"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
