@@ -2,6 +2,7 @@ from host import HostAccount
 from species import Species
 from genome import Genome
 from compara import Compara
+from util import NoItemError
 
 __all__ = ['assembly', 'compara', 'database', 'genome', 'host', 'name',
            'region', 'related_region', 'sequence', 'species', 'util',
@@ -11,7 +12,7 @@ __author__ = "Gavin Huttley"
 __copyright__ = "Copyright 2007, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Hua Ying"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Gavin Huttley"
 __email__ = "Gavin.Huttley@anu.edu.au"
 __status__ = "alpha"
