@@ -13,7 +13,7 @@ __author__ = "Gavin Huttley"
 __copyright__ = "Copyright 2007, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Hua Ying"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Gavin Huttley"
 __email__ = "Gavin.Huttley@anu.edu.au"
 __status__ = "alpha"
@@ -47,8 +47,10 @@ class Compara(object):
             setattr(self, common, genome)
     
     def __str__(self):
-        return "Compara(Species=%s; Release=%s; connected=%s)" % \
-                    (self.Species, self.Release, self.ComparaDb is not None)
+        my_type = self.__class__.__name__
+        return "%s(Species=%s; Release=%s; connected=%s)" % \
+                    (my_type, self.Species, self.Release,
+                    self.ComparaDb is not None)
     
     def _connect_db(self):
         # TODO can the connection be all done in init?
