@@ -15,7 +15,7 @@ __author__ = "Gavin Huttley"
 __copyright__ = "Copyright 2007, The Cogent Project"
 __credits__ = ["Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Gavin Huttley"
 __email__ = "Gavin.Huttley@anu.edu.au"
 __status__ = "alpha"
@@ -88,7 +88,8 @@ class Genome(object):
         self._feature_coord_levels = FeatureCoordLevels(self.Species)
     
     def __str__(self):
-        return "Genome(Species='%s'; Release='%s')" % (self.Species,
+        my_type = self.__class__.__name__
+        return "%s(Species='%s'; Release='%s')" % (my_type, self.Species,
                                                        self.Release)
     
     def __repr__(self):
@@ -126,21 +127,27 @@ class Genome(object):
     
     OtherFeaturesDb = property(_get_other_db)
     
-    def _get_biotype_description_condition(self, gene_table, Description=None, BioType=None):
+    def _get_biotype_description_condition(self, gene_table, Description=None, BioType=None, like=True):
         assert Description or BioType, "no valid argument provided"
-        btype_like, descr_like = None, None
+        btype, descr = None, None
         
         if BioType:
-            btype_like = gene_table.c.biotype.like('%'+BioType+'%')
+            if like:
+                btype = gene_table.c.biotype.like('%'+BioType+'%')
+            else:
+                btype = gene_table.c.biotype==BioType
         if Description:
-            descr_like = gene_table.c.description.like('%'+Description+'%')
+            if like:
+                descr = gene_table.c.description.like('%'+Description+'%')
+            else:
+                descr = gene_table.c.description==Description
         
-        if btype_like and descr_like:
-            condition = sql.and_(btype_like, descr_like)
-        elif btype_like:
-            condition = btype_like
-        elif descr_like:
-            condition = descr_like
+        if btype and descr:
+            condition = sql.and_(btype, descr)
+        elif btype:
+            condition = btype
+        elif descr:
+            condition = descr
         
         return condition
     
@@ -148,13 +155,13 @@ class Genome(object):
         join_obj = gene_id_table.join(gene_table, gene_id_table.c.gene_id==gene_table.c.gene_id)
         select_obj = [gene_id_table.c.stable_id, gene_table]
         if db.Type == 'core':
-            join_obj = join_obj.join(xref_table, gene_table.c.display_xref_id==xref_table.c.xref_id)
+            join_obj = join_obj.outerjoin(xref_table, gene_table.c.display_xref_id==xref_table.c.xref_id)
             select_obj.append(xref_table.c.display_label)
         query = sql.select(select_obj, from_obj=[join_obj], whereclause=condition)
         return query
     
     def _get_gene_query(self, db, Symbol=None, Description=None, StableId=None,
-                         BioType=None):
+                         BioType=None, like=True):
         xref_table = [None, db.getTable('xref')][db.Type == 'core']
         gene_table = db.getTable('gene')
         gene_id_table = db.getTable('gene_stable_id')
@@ -165,7 +172,7 @@ class Genome(object):
         elif StableId:
             condition = gene_id_table.c.stable_id==StableId
         else:
-            condition = self._get_biotype_description_condition(gene_table, Description, BioType)
+            condition = self._get_biotype_description_condition(gene_table, Description, BioType, like)
         
         query = self._build_gene_query(db, condition, gene_table, gene_id_table, xref_table)
         
@@ -178,7 +185,7 @@ class Genome(object):
                             Strand=Strand, ensembl_coord=ensembl_coord)
     
     def getGenesMatching(self, Symbol=None, Description=None, StableId=None,
-                         BioType=None):
+                         BioType=None, like=True):
         """Symbol: HGC gene symbol, case doesn't matter
         description: a functional description
         StableId: the ensebl identifier
@@ -186,7 +193,6 @@ class Genome(object):
         # TODO additional arguments to satisfy: external_ref, go_terms
         if Symbol is not None:
             Symbol = Symbol.lower()
-        
         # biotype -> gene
         # description -> gene
         # Symbols -> xref
@@ -200,7 +206,7 @@ class Genome(object):
         # TODO catch codnitions where user passes in both a symbol and a
         # biotype
         args = dict(Symbol=Symbol, Description=Description, 
-                     StableId=StableId, BioType=BioType)
+                     StableId=StableId, BioType=BioType, like=like)
         query = self._get_gene_query(self.CoreDb, **args)
         records = query.execute()
         for record in records:
@@ -334,16 +340,11 @@ class Genome(object):
     def _feature_coord_levels(self):
         if str(self._feature_coord_levels):
             return self._feature_coord_levels
-        feature_types = ['gene', 'est', 'variation']
+        feature_types = ['gene', 'est', 'variation', 'cpg', 'repeat']
         feature_coord_levels = self._get_feature_coord_levels(feature_types)
         return self._feature_coord_levels
     
     FeatureCoordLevels = property(_feature_coord_levels)
-    def setFeatureCoordLevel(self, feature_type, feature_coord_level):
-        feature_coord_levels = self._get_feature_coord_levels([feature_type])
-        availables = feature_coord_levels[feature_type].levels.split(', ')
-        assert feature_coord_level in availables, 'Invalid Coord system name: %s'%feature_coord_level
-        feature_coord_levels[feature_type].level_in_use = feature_coord_level
     
     def getFeatures(self, region=None, feature_types=None, where_feature=None,
                     CoordName=None, Start=None, End=None, Strand=None,
@@ -381,14 +382,15 @@ class Genome(object):
         
         for feature_type in feature_types:
             target_func, target_class = target_coords_funcs[feature_type]
-            feature_coord = feature_coord_levels[feature_type].level_in_use
             db = [self.CoreDb, self.OtherFeaturesDb][feature_type == 'est']
-            chrom_other_coords = get_coord_conversion(coord, feature_coord,
-                                        db, where=where_feature)
-            for chrom_coord, other_coord in chrom_other_coords:
-                for region in target_func(db, target_class, chrom_coord,
-                                        other_coord, where_feature):
-                    yield region
+            feature_coords = feature_coord_levels[feature_type].levels
+            for feature_coord in feature_coords:
+                chrom_other_coords = get_coord_conversion(coord, feature_coord,
+                                            db, where=where_feature)
+                for chrom_coord, other_coord in chrom_other_coords:
+                    for region in target_func(db, target_class, chrom_coord,
+                                            other_coord, where_feature):
+                        yield region
     
     def getVariation(self, Effect=None, Symbol=None, like=True,
                      validated=False):
