@@ -16,14 +16,14 @@ __author__ = "Matthew Wakefield"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Matthew Wakefield", "Peter Maxwell", "Gavin Huttley"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Matthew Wakefield"
 __email__ = "matthew.wakefield@wehi.edu.au"
 __status__ = "Production"
 
 nucleotide_models = ['JC69','F81','HKY85', 'GTR']
 
-codon_models = ['MG94HKY', 'MG94GTR', 'GY94', 'H04G', 'H04GK', 'H04GGK']
+codon_models = ['CNFGTR', 'CNFHKY', 'MG94HKY', 'MG94GTR', 'GY94', 'H04G', 'H04GK', 'H04GGK']
 
 protein_models = [ 'DSO78', 'AH96', 'AH96_mtmammals', 'JTT92', 'WG01']
 
@@ -86,6 +86,39 @@ def GTR():
     
 
 # Codon Models
+def CNFGTR():
+    """Conditional nucleotide frequency codon substitution model, GTR variant
+    (with params analagous to the nucleotide GTR model).
+    
+    See Yap, Lindsay, Easteal and Huttley, Mol Biol Evol, In press."""
+    return substitution_model.Codon(
+            motif_probs = None,
+            do_scaling = True,
+            model_gaps = False,
+            recode_gaps = True,
+            name = 'CNFGTR',
+            predicates = _gtr_preds+[_omega],
+            mprob_model='conditional'
+            )
+        
+    
+
+def CNFHKY():
+    """Conditional nucleotide frequency codon substitution model, HKY variant
+    (with kappa, the ratio of transitions to transversions)
+    
+    See Yap, Lindsay, Easteal and Huttley, Mol Biol Evol, In press."""
+    return substitution_model.Codon(
+            motif_probs = None,
+            do_scaling = True,
+            model_gaps = False,
+            recode_gaps = True,
+            name = 'CNFHKY',
+            predicates = [_kappa, _omega],
+            mprob_model='conditional'
+            )
+    
+
 def MG94HKY():
     """Muse and Gaut 1994 codon substitution model, HKY variant (with kappa,
     the ratio of transitions to transversions)
@@ -137,6 +170,7 @@ def Y98():
                 'kappa' : 'transition',
                 'omega' : 'replacement',
                 },
+            mprob_model = 'tuple',
             )
     
 
@@ -152,6 +186,7 @@ def H04G():
             recode_gaps = True,
             name = 'H04G',
             predicates = [_cg, _kappa, _omega],
+            mprob_model = 'tuple',
             )
     
 
@@ -167,6 +202,7 @@ def H04GK():
             recode_gaps = True,
             name = 'H04GK',
             predicates = [_cg_k, _kappa, _omega],
+            mprob_model = 'tuple',
             )
     
 
@@ -182,6 +218,7 @@ def H04GGK():
             recode_gaps = True,
             name = 'H04GGK',
             predicates = [_cg, _cg_k, _kappa, _omega],
+            mprob_model = 'tuple',
             )
     
 
