@@ -7,13 +7,14 @@ Current formats include restructured text (keyed by 'rest'), latex, html,
 columns separated by a provided string, and a simple text format.
 """
 import textwrap
+from cogent.util.warning import discontinued
 
 __author__ = "Gavin Huttley"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Peter Maxwell", "Matthew Wakefield",
                     "Jeremy Widmann"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -35,6 +36,78 @@ def html(text, **kwargs):
     # assuming run from the correct directory
     return publish_string(source=text, writer_name='html', **kwargs)
 
+def _merge_cells(row):
+    """merges runs of identical row cells.
+    
+    returns a list with structure [((span_start, span_end), cell value),..]"""
+    new_row = []
+    last = 0
+    span = 1 # the minimum
+    for i in range(1, len(row), 1):
+        if row[i-1] != row[i]:
+            new_row.append(((last,last+span), row[i-1]))
+            last=i
+            span=1
+            continue
+        span += 1
+    
+    new_row.append(((last,last+span), row[i]))
+    return new_row
+
+def rich_html(rows, row_cell_func=None, header=None, header_cell_func=None,
+    element_formatters={}, merge_identical=True, compact=True):
+    """returns just the html Table string
+    
+    Arguments:
+        - rows: table rows
+        - row_cell_func: callback function that formats the row values. Must
+          take the row value and coordinates (row index, column index).
+        - header: the table header
+        - header_cell_func: callback function that formats the column headings
+          must take the header label value and coordinate
+        - element_formatters: a dictionary of specific callback funcs for
+          formatting individual html table elements.
+          e.g. {'table': lambda x: '<table border="1" class="docutils">'}
+        - merge_identical: cells within a row are merged to one span.
+    
+    Note: header_cell_func and row_cell_func override element_formatters.
+    """
+    formatted = element_formatters.get
+    data = [formatted('table', '<table>')]
+    # TODO use the docutils writer html convertor instead of str, for correct
+    # escaping of characters
+    if row_cell_func is None:
+        row_cell_func = lambda v,r,c: '<td>%s</td>' % v
+    
+    if header_cell_func is None:
+        header_cell_func = lambda v, c: '<th>%s</th>' % v
+    
+    if merge_identical:
+        row_iterator = _merge_cells
+    else:
+        row_iterator = enumerate
+    
+    if header:
+        th = formatted('th', '<th>')
+        row = [header_cell_func(label, i) for i, label in enumerate(header)]
+        data += [formatted('tr', '<tr>')]+row+['</tr>']
+    
+    formatted_rows = []
+    td = formatted('td', '<td>')
+    for ridx, row in enumerate(rows):
+        new = [formatted('tr', '<tr>')]
+        for cidx, cell in row_iterator(row):
+            new += [row_cell_func(cell, ridx, cidx)]
+        new += ['</tr>']
+        formatted_rows += new
+    data += formatted_rows
+    data += ['</table>']
+    if compact:
+        data = ''.join(data)
+    else:
+        data = '\n'.join(data)
+    return data
+
 def latex(rows, header=None, caption=None, justify=None, label=None, position = None):
     """Returns the text a LaTeX longtable.
     
@@ -320,6 +393,7 @@ def asReportlabTable(header, formatted_table, total_width=476, table_style = Non
         - total_width: table width
         - table_style: reportlab compliant table style settings.
     """
+    discontinued('function', 'asReportlabTable', 1.5)
     from reportlab.platypus import Table as reportlabTable
     from reportlab.lib import colors
     if not table_style:
@@ -383,9 +457,11 @@ def formattedCells(rows, header = None, digits=4, column_templates = None, missi
             except IndexError:
                 entry = '%s' % missing_data
             else:
-                if not entry and not \
-                (isinstance(entry, float) or isinstance(entry, int)):
-                    entry = '%s' % missing_data
+                if not entry:
+                    try:
+                        float(entry) # could numerically be 0, so not missing
+                    except (ValueError, TypeError):
+                        entry = '%s' % missing_data
             
             # attempt formatting
             if col_head in column_templates:
