@@ -11,6 +11,7 @@ __all__ = ['function_optimisation',
            'scipy_optimize',
            'simannealingoptimiser',
            'solve',
+           'spatial',
            'svd']
 
 
@@ -19,7 +20,7 @@ __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Peter Maxwell", "Matthew Wakefield",
                     "Rob Knight", "Edward Lang", "Sandra Smit"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
