@@ -4,7 +4,8 @@
 Also provides NumberList and FrequencyDistribution, two classes for
 working with statistical data.
 """
-__all__ = ['distribution', 'histogram', 'kendall', 'ks', 'special', 'test',
+__all__ = ['alpha_diversity', 'distribution', 'histogram', 'kendall', 'ks', 
+        'rarefaction', 'special', 'test',
            'util']
 
 # GAH: this is a temporary introduction, so users get notice of structure change and
@@ -16,7 +17,7 @@ __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Rob Knight", "Sandra Smit",
                     "Catherine Lozupone", "Micah Hamady"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
