@@ -4,9 +4,9 @@ __all__ = ['fast_tree',
            'fast_unifrac' ] 
 __author__ = ""
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
-__credits__ = ["Rob Knight", "Micah Hamady"]
-__license__ = "All rights reserved"
-__version__ = "1.3.1"
+__credits__ = ["Rob Knight", "Micah Hamady", "Justin Kuczynski"]
+__license__ = "GPL"
+__version__ = "1.4"
 __maintainer__ = "Micah Hamady"
 __email__ = "hamady@colorado.edu"
 __status__ = "Prototype"
