@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 """Code for dealing with molecular structures."""
-__all__ = ['knots', 'pairs_util', 'rna2d']
+__all__ = ['knots', 'pairs_util', 'rna2d', 'selection', 'annotation',
+           'manipulation']
 
 __author__ = "Rob Knight"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
-__credits__ = ["Rob Knight", "Sandra Smit", "Peter Maxwell"]
+__credits__ = ["Rob Knight", "Sandra Smit", "Peter Maxwell", "Marcin Cieslik"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
