@@ -7,7 +7,7 @@ Current output formats include pickle (pythons serialisation format), restructur
 Table can read pickled and delimited formats.
 
 """
-
+from __future__ import division
 import cPickle, csv
 import numpy
 from cogent.format import table as table_format
@@ -18,7 +18,7 @@ __author__ = "Gavin Huttley"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Gavin Huttley", "Felix Schill"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -64,11 +64,11 @@ class _Header(list):
     
     def __setslice__(self, *args):
         """disallowed"""
-        raise RuntimeError("Table Header is immutable, use withNewColumns")
+        raise RuntimeError("Table Header is immutable, use withNewHeader")
     
     def __setitem__(self, *args):
         """disallowed"""
-        raise RuntimeError("Table Header is immutable, use withNewColumns")
+        raise RuntimeError("Table Header is immutable, use withNewHeader")
     
 
 class Table(DictArray):
@@ -239,9 +239,9 @@ class Table(DictArray):
         
         Arguments:
             - format: possible formats are 'rest', 'latex', 'html', 'phylip', or
-             simple text (default)
+              simple text (default)
             - sep: A string separator for delineating columns, e.g. ',' or '\t'.
-            Overrides format.
+              Overrides format.
         """
         if format.lower() == 'phylip':
             missing_data = "%.4f" % 0.0
@@ -249,7 +249,7 @@ class Table(DictArray):
             missing_data = self._missing_data
         
         # convert self to a 2D list
-        formatted_table = [row.array.tolist() for row in self]
+        formatted_table = self.array.tolist()
         header, formatted_table = table_format.formattedCells(formatted_table,
                                     self.Header,
                                     digits = self._digits,
@@ -278,6 +278,34 @@ class Table(DictArray):
             return table_format.simpleFormat(*args + (self._max_width,
                                 self._row_ids, borders, self.Space))
     
+    def toRichHtmlTable(self, row_cell_func=None, header_cell_func=None,
+                element_formatters={}, merge_identical=True, compact=True):
+        """returns just the table html code.
+        Arguments:
+            - row_cell_func: callback function that formats the row values. Must
+              take the row value and coordinates (row index, column index).
+            - header_cell_func: callback function that formats the column headings
+              must take the header label value and coordinate
+            - element_formatters: a dictionary of specific callback funcs for
+              formatting individual html table elements.
+              e.g. {'table': lambda x: '<table border="1" class="docutils">'}
+            - merge_identical: cells within a row are merged to one span.
+        """
+        formatted_table = self.array.tolist()
+        header, formatted_table = table_format.formattedCells(formatted_table,
+                                    self.Header,
+                                    digits = self._digits,
+                                    column_templates = self._column_templates,
+                                    missing_data = self._missing_data)
+        # but we strip the cell spacing
+        header = [v.strip() for v in header]
+        rows = [[c.strip() for c in r] for r in formatted_table]
+        return table_format.rich_html(rows, row_cell_func=row_cell_func,
+                                      header=header,
+                                      header_cell_func=header_cell_func,
+                                      element_formatters=element_formatters,
+                                      compact=compact)
+    
     def writeToFile(self, filename, mode = 'w', writer = None, format = None,
                  sep = None, **kwargs):
         """Write table to filename in the specified format. If a format is not
@@ -367,21 +395,19 @@ class Table(DictArray):
             - columns: if None, all data are returned"""
         
         if columns is None:
-            columns = self.Header[:]
+            return self.array.tolist()
         
         if isinstance(columns, str):
             # assumes all column headings are strings.
             columns = (columns,)
         
-        new_twoD = []
-        for row in self:
-            new_row = [row[entry] for entry in columns]
-            new_twoD.append(new_row)
+        column_indices = map(self.Header.index, columns)
+        result = self.array.take(column_indices, axis=1)
         
         if len(columns) == 1:
-            new_twoD = [cell[0] for cell in new_twoD]
+            result = result.flatten()
         
-        return new_twoD
+        return result.tolist()
     
     def _callback(self, callback, row, columns=None, num_columns=None):
         if callable(callback):
@@ -423,6 +449,22 @@ class Table(DictArray):
         kw.update(kwargs)
         return Table(header = self.Header, rows = sub_set, **kw)
     
+    def filteredByColumn(self, callback, **kwargs):
+        """Returns a table with columns identified by callback
+        
+        Arguments:
+            - callback: A function which takes the columns delimited
+            by columns and returns True/False, or a string representing valid
+            python code to be evaluated."""
+        data = self.array.transpose()
+        column_indices = []
+        append = column_indices.append
+        for index, row in enumerate(data):
+            if callback(row):
+                append(index)
+        columns = numpy.take(self.Header, column_indices)
+        return self.getColumns(columns, **kwargs)
+    
     def count(self, callback, columns=None, **kwargs):
         """Returns number of rows for which the provided callback
         function returns True when passed row data from columns. Row data
@@ -573,13 +615,13 @@ class Table(DictArray):
     
     def getDistinctValues(self, column):
         """returns the set of distinct values for the named column(s)"""
-        vals = set()
         columns = [column, [column]][type(column) == str]
-        for row in self:
-            val = [row[column] for column in columns]
-            val = [val, [tuple(val)]][len(columns) > 1]
-            vals.update(val)
-        return vals
+        data = self.getRawData(column)
+        
+        if len(columns) > 1:
+            data = [tuple(row) for row in data]
+        
+        return set(data)
     
     def joined(self, other_table, columns_self=None, columns_other=None,
                 inner_join=True, **kwargs):
@@ -694,7 +736,111 @@ class Table(DictArray):
         
         return Table(header=new_header, rows=joined_table, **kwargs)
     
-    def summed(self, column):
-        """returns the sum of row values for column"""
-        vals = [r[column] for r in self]
-        return sum(vals)
+    def summed(self, indices=None, col_sum=True, strict=True, **kwargs):
+        """returns the sum of numerical values for column(s)/row(s)
+        
+        Arguments:
+            - indices: column name(s) or indices or row indices
+            - col_sum: sums values in the indicated column, the default. If
+              False, returns the row sum.
+            - strict: if False, ignores cells with non-numeric data in the
+              column/row."""
+        
+        all = indices is None
+        
+        if type(indices) == str:
+            assert col_sum, "Must use row integer indices"
+            indices = self.Header.index(indices)
+        elif type(indices) == int: # a single index
+            indices = [indices]
+        elif not all:
+            raise RuntimeError("unknown indices type: %s" % indices)
+        
+        if not all:
+            vals = self.array.take([indices], axis=[0,1][col_sum]).flatten()
+            if strict:
+                result = vals.sum()
+            else:
+                result = sum(v for v in vals if type(v)!=str)
+        else:
+            # a multi-rowed result
+            if col_sum:
+                vals = self.array
+            else:
+                vals = self.array.transpose() 
+            
+            if strict:
+                result = vals.sum(axis=0).tolist()
+            else:
+                result = []
+                append = result.append
+                # we need to iterate over the elements to be summed, so we
+                # have to transpose
+                for row in vals.transpose():
+                    try:
+                        num = row.sum()
+                    except TypeError:
+                        num = sum(r for r in row if type(r) != str)
+                    append(num)
+            
+        
+        return result
+    
+    def normalized(self, by_row=True, denominator_func=None, **kwargs):
+        """returns a table with elements expressed as a fraction according
+        to the results from func
+        
+        Arguments:
+            - by_row: normalisation done by row
+            - denominator_func: a callback function that takes an array and
+              returns a value to be used as the denominator. Default is sum."""
+        
+        if denominator_func:
+            data = self.array
+            if not by_row:
+                data = data.transpose()
+            denominators = [denominator_func(row) for row in data]
+        else:
+            denominators = self.summed(col_sum=not by_row)
+        
+        if by_row:
+            values = self.array
+        else:
+            values = self.array.transpose()
+        
+        rows = [values[i]/denom for i, denom in enumerate(denominators)]
+        rows = numpy.array(rows)
+        
+        if not by_row:
+            rows = rows.transpose()
+        
+        return Table(header=self.Header, rows=rows, **kwargs)
+    
+    def transposed(self, new_column_name, select_as_header=None, **kwargs):
+        """returns the transposed table.
+        
+        Arguments:
+            - new_column_name: the existing header will become a column with
+              this name
+            - select_as_header: current column name containing data to be used
+              as the header. Defaults to the first column.
+        """
+        select_as_header = select_as_header or self.Header[0]
+        assert select_as_header in self.Header, \
+                    '"%s" not in table Header' % select_as_header
+        
+        raw_data = self.getRawData()
+        raw_data.insert(0, self.Header)
+        transposed = numpy.transpose(raw_data)
+        
+        # indices for the header and non header rows
+        header_index = self.Header.index(select_as_header)
+        
+        data_indices = range(0, header_index)+range(header_index+1,
+                                                    len(transposed))
+        
+        header = list(numpy.take(transposed, [header_index], axis=0)[0])
+        header = [new_column_name]+header[1:] # [1:] slice excludes old name
+        rows = numpy.take(transposed, data_indices, axis=0)
+        return Table(header=header, rows=rows, **kwargs)
+    
