@@ -11,7 +11,8 @@ from cogent.app.infernal import Cmalign, Cmbuild, Cmcalibrate, Cmemit, Cmscore,\
     Cmsearch, Cmstat, cmbuild_from_alignment, cmbuild_from_file, \
     cmalign_from_alignment, cmalign_from_file, cmsearch_from_alignment,\
     cmsearch_from_file
-from cogent.parse.rfam import MinimalRfamParser
+from cogent.parse.rfam import MinimalRfamParser, ChangedRnaSequence, \
+    ChangedSequence
 from cogent.format.stockholm import stockholm_from_alignment
 from cogent.struct.rna2d import ViennaStructure, wuss_to_vienna
 
@@ -19,7 +20,7 @@ __author__ = "Jeremy Widmann"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Jeremy Widmann"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Jeremy Widmann"
 __email__ = "jeremy.widmann@colorado.edu"
 __status__ = "Development"
