@@ -5,6 +5,7 @@ __all__ = ['test_alignment',
            'test_annotation',
            'test_bitvector',
            'test_core_standalone',
+           'test_entity',
            'test_genetic_code',
            'test_info',
            'test_location',
@@ -19,10 +20,10 @@ __all__ = ['test_alignment',
 __author__ = ""
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Catherine Lozupone", "Peter Maxwell", "Rob Knight",
-                    "Gavin Huttley", "Jeremy Widmann", "Greg Caporaso", 
-                    "Sandra Smit"]
+                    "Gavin Huttley", "Jeremy Widmann", "Greg Caporaso",
+                    "Sandra Smit", "Justin Kuczynski", "Marcin Cieslik"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
