@@ -26,7 +26,7 @@ __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Jeremy Widmann", "Catherine Lozuopone", "Gavin Huttley",
                     "Rob Knight", "Daniel McDonald"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
@@ -901,7 +901,21 @@ class SequenceCollectionBaseTests(object):
         seqs = self.Class(data=[("a", dna),("b", dna)])
         self.assertEqual(seqs.degap().toFasta(), expect)
     
-
+    def test_padSeqs(self):
+        """SequenceCollection padSeqs should work on alignment."""
+        #pad to max length
+        padded1 = self.ragged_padded.padSeqs()
+        seqs1 = list(padded1.iterSeqs(seq_order=['a','b','c']))
+        self.assertEqual(map(str,seqs1),['AAAAAA', 'AAA---', 'AAAA--'])
+        
+        #pad to alternate length
+        padded1 = self.ragged_padded.padSeqs(pad_length=10)
+        seqs1 = list(padded1.iterSeqs(seq_order=['a','b','c']))
+        self.assertEqual(map(str,seqs1),['AAAAAA----', 'AAA-------',\
+            'AAAA------'])
+        
+        #assertRaises error when pad_length is less than max seq length
+        self.assertRaises(ValueError, self.ragged_padded.padSeqs, 5)
 
 class SequenceCollectionTests(SequenceCollectionBaseTests, TestCase):
     """Tests of the SequenceCollection object. Includes ragged collection tests.
@@ -947,6 +961,22 @@ class SequenceCollectionTests(SequenceCollectionBaseTests, TestCase):
 
 
         self.assertRaises(ValueError,  align_rag.toPhylip)
+    
+    def test_padSeqs_ragged(self):
+        """SequenceCollection padSeqs should work on ragged alignment."""
+        #pad to max length
+        padded1 = self.ragged.padSeqs()
+        seqs1 = list(padded1.iterSeqs(seq_order=['a','b','c']))
+        self.assertEqual(map(str,seqs1),['AAAAAA', 'AAA---', 'AAAA--'])
+        
+        #pad to alternate length
+        padded1 = self.ragged.padSeqs(pad_length=10)
+        seqs1 = list(padded1.iterSeqs(seq_order=['a','b','c']))
+        self.assertEqual(map(str,seqs1),['AAAAAA----', 'AAA-------',\
+            'AAAA------'])
+        
+        #assertRaises error when pad_length is less than max seq length
+        self.assertRaises(ValueError, self.ragged.padSeqs, 5)
 
 class AlignmentBaseTests(SequenceCollectionBaseTests):
     """Tests of basic Alignment functionality. All Alignments should pass these.
@@ -1425,7 +1455,16 @@ class AlignmentTests(AlignmentBaseTests, TestCase):
         self.assertEqual(result[2].todict(), {'seq3': 'GTACG', 'seq2': 'GTACG', 'seq1': 'GTACG'})
         self.assertEqual(result[3].todict(), {'seq3': 'TACGT', 'seq2': 'TACGT', 'seq1': 'TACGT'})
  
-
+    def test_withGapsFrom(self):
+        """withGapsFrom should overwrite with gaps."""
+        gapless   = self.Class({'seq1': 'TCG', 'seq2': 'TCG'})
+        pregapped = self.Class({'seq1': '-CG', 'seq2': 'TCG'})
+        template  = self.Class({'seq1': 'A-?', 'seq2': 'ACG'})
+        r1 = gapless.withGapsFrom(template).todict()
+        r2 = pregapped.withGapsFrom(template).todict()
+        self.assertEqual(r1, {'seq1': 'T-G', 'seq2': 'TCG'}) 
+        self.assertEqual(r2, {'seq1': '--G', 'seq2': 'TCG'}) 
+        
 class DenseAlignmentSpecificTests(TestCase):
     """Tests of the DenseAlignment object and its methods"""
 
