@@ -1,5 +1,5 @@
 #/usr/bin/env python
-from cogent.draw.matplotlib.arrow_rates import make_arrow_plot, sample_data
+from cogent.draw.arrow_rates import make_arrow_plot, sample_data
 from cogent.util.unit_test import TestCase, main
 from os import remove
    
@@ -7,7 +7,7 @@ __author__ = "Rob Knight"
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Rob Knight"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
@@ -21,7 +21,7 @@ class arrow_rates_tests(TestCase):
         """arrow_plot should write correct file and not raise exception"""
         make_arrow_plot(sample_data, graph_name='arrows.png')
         #comment out line below to see the result
-        remove('arrows.png')
+        #remove('arrows.png')
 
 if __name__ == '__main__':
     main()
