@@ -31,7 +31,7 @@ __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Peter Maxwell", "Gavin Huttley", "Rob Knight",
                     "Matthew Wakefield", "Brett Easton"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Gavin Huttley"
 __email__ = "gavin.huttley@anu.edu.au"
 __status__ = "Production"
@@ -144,7 +144,7 @@ class LikelihoodCalcs(TestCase):
         """just rate is gamma distributed"""
         submod = substitution_model.Codon(
             predicates={'kappa': 'transition', 'omega': 'replacement'},
-            ordered_param='rate', distribution='gamma')
+            ordered_param='rate', distribution='gamma', mprob_model='tuple')
         lf = self._makeLikelihoodFunction(submod, self.alignment, bins=3)
         try:
             values = lf.getParamValueDict(['bin'])['omega_factor'].values()
@@ -161,7 +161,8 @@ class LikelihoodCalcs(TestCase):
         """rate is gamma distributed omega follows"""
         submod = substitution_model.Codon(
             predicates={'kappa': 'transition', 'omega': 'replacement'},
-            ordered_param='rate', partitioned_params='omega', distribution='gamma')
+            ordered_param='rate', partitioned_params='omega', 
+            distribution='gamma', mprob_model='tuple')
         lf = self._makeLikelihoodFunction(submod, self.alignment,bins=3) 
         values = lf.getParamValueDict(['bin'])['omega_factor'].values()
         self.assertEqual(round(sum(values) / len(values), 6), 1.0)
@@ -171,7 +172,8 @@ class LikelihoodCalcs(TestCase):
     def test_binned_partition(self):
         submod = substitution_model.Codon(
             predicates={'kappa': 'transition', 'omega': 'replacement'},
-            ordered_param='rate', partitioned_params='omega', distribution='free')
+            ordered_param='rate', partitioned_params='omega', 
+            distribution='free', mprob_model='tuple')
         lf = self._makeLikelihoodFunction(submod, self.alignment, bins=3)
         values = lf.getParamValueDict(['bin'])['omega_factor'].values()
         self.assertEqual(round(sum(values) / len(values), 6), 1.0)
@@ -180,7 +182,8 @@ class LikelihoodCalcs(TestCase):
     def test_complex_binned_partition(self):
         submod = substitution_model.Codon(
             predicates={'kappa': 'transition', 'omega': 'replacement'},
-            ordered_param='kappa', partitioned_params=['omega'])
+            ordered_param='kappa', partitioned_params=['omega'], 
+            mprob_model='tuple')
         lf = self._makeLikelihoodFunction(submod, self.alignment,
                     bins=['slow', 'fast'])
         lf.setParamRule('kappa', value=1.0, is_const=True)
@@ -192,60 +195,68 @@ class LikelihoodCalcs(TestCase):
     def test_codon(self):
         """test a three taxa codon model."""
         submod = substitution_model.Codon(
+            equal_motif_probs=True,
             do_scaling=False,
             motif_probs=None,
-            predicates={'kappa': 'transition', 'omega': 'replacement'})
+            predicates={'kappa': 'transition', 'omega': 'replacement'},
+            mprob_model='tuple')
         
         self.par_values.update({'omega':0.5})
-        likelihood_function = self._makeLikelihoodFunction(submod, self.alignment)
+        likelihood_function = self._makeLikelihoodFunction(
+                submod, self.alignment)
                     
         for par, val in self.par_values.items():
             likelihood_function.setpar(par, val)
             
         likelihood_function.setpar("length", self.length)
         evolve_lnL = likelihood_function.testfunction()
-        self.assertEqual("%.6f" % -57.8379659216, "%.6f" % evolve_lnL)
+        self.assertFloatEqual(evolve_lnL, -80.67069614541883)
     
     def test_nucleotide(self):
         """test a nucleotide model."""
         submod = Nucleotide(
+            equal_motif_probs=True,
             do_scaling=False,
             motif_probs=None,
             predicates={'kappa': 'transition'})
         # now do using the evolve
-        likelihood_function = self._makeLikelihoodFunction(submod, self.alignment)
+        likelihood_function = self._makeLikelihoodFunction(
+                submod, self.alignment)
         for par, val in self.par_values.items():
             likelihood_function.setpar(par, val)
             
         likelihood_function.setpar("length", self.length)
         evolve_lnL = likelihood_function.testfunction()
-        self.assertEqual("%.6f" % -155.775725365, "%.6f" % evolve_lnL)
+        self.assertFloatEqual(evolve_lnL, -157.49363874840455)
     
     def test_dinucleotide(self):
         """test a dinucleotide model."""
         submod = substitution_model.Dinucleotide(
+                equal_motif_probs=True,
                 do_scaling=False,
                 motif_probs = None,
-                predicates = {'kappa': 'transition'})
-        likelihood_function = self._makeLikelihoodFunction(submod, self.alignment)
+                predicates = {'kappa': 'transition'},
+                mprob_model='tuple')
+        likelihood_function = self._makeLikelihoodFunction(
+                submod, self.alignment)
         for par, val in self.par_values.items():
             likelihood_function.setpar(par, val)
             
         likelihood_function.setpar("length", self.length)
         evolve_lnL = likelihood_function.testfunction()
-        self.assertEqual("%.6f" % -85.2399172216, "%.6f" % evolve_lnL)
+        self.assertFloatEqual(evolve_lnL, -102.48145536663735)
     
     def test_protein(self):
         """test a protein model."""
         submod = substitution_model.Protein(
-            do_scaling=False, motif_probs=None)
+            do_scaling=False, equal_motif_probs=True)
         alignment = self.alignment.getTranslation()
         
         likelihood_function = self._makeLikelihoodFunction(submod, alignment)
         
         likelihood_function.setpar("length", self.length)
         evolve_lnL = likelihood_function.testfunction()
-        self.assertEqual("%.6f" % -76.301896714, "%.6f" % evolve_lnL)
+        self.assertFloatEqual(evolve_lnL, -89.830370754876185)
     
 
 class LikelihoodFunctionTests(TestCase):
@@ -264,8 +275,8 @@ class LikelihoodFunctionTests(TestCase):
         self.tree = LoadTree(
                 filename = os.path.join(data_path, 'brca1_5.tree'))
     
-    def _makeLikelihoodFunction(self):
-        lf = self.submodel.makeLikelihoodFunction(self.tree)
+    def _makeLikelihoodFunction(self, **kw):
+        lf = self.submodel.makeLikelihoodFunction(self.tree, **kw)
         lf.setParamRule('beta', is_independent=True)
         lf.setAlignment(self.data)
         return lf
@@ -317,6 +328,29 @@ motif    mprobs
     G    0.2500
 ---------------""")
     
+        likelihood_function = self._makeLikelihoodFunction(digits=2,space=2)
+        self.assertEqual(str(likelihood_function), \
+"""Likelihood Function Table\n\
+===============================
+     edge  parent  length  beta
+-------------------------------
+    Human  edge.0    1.00  1.00
+HowlerMon  edge.0    1.00  1.00
+   edge.0  edge.1    1.00  1.00
+    Mouse  edge.1    1.00  1.00
+   edge.1    root    1.00  1.00
+NineBande    root    1.00  1.00
+ DogFaced    root    1.00  1.00
+-------------------------------
+=============
+motif  mprobs
+-------------
+    T    0.25
+    C    0.25
+    A    0.25
+    G    0.25
+-------------""")
+    
     def test_calclikelihood(self):
         likelihood_function = self._makeLikelihoodFunction()
         self._setLengthsAndBetas(likelihood_function)
@@ -369,7 +403,7 @@ motif    mprobs
         "Simulate alignment with dinucleotide model"
         al = LoadSeqs(data={'a':'ggaatt','c':'cctaat'})
         t = LoadTree(treestring="(a,c);")
-        sm = substitution_model.Dinucleotide()
+        sm = substitution_model.Dinucleotide(mprob_model='tuple')
         pc = sm.makeParamController(t)
         lf = pc.makeCalculator(al)
         simalign = lf.simulateAlignment()
@@ -402,7 +436,7 @@ motif    mprobs
         def use_root_seq(root_sequence):
             al = LoadSeqs(data={'a':'ggaatt','c':'cctaat'})
             t = LoadTree(treestring="(a,c);")
-            sm = substitution_model.Dinucleotide()
+            sm = substitution_model.Dinucleotide(mprob_model='tuple')
             pc = sm.makeParamController(t)
             lf = pc.makeCalculator(al)
             simalign = lf.simulateAlignment(exclude_internal=False,
@@ -546,200 +580,6 @@ motif    mprobs
                         edges=['NineBande', 'DogFaced'], is_clade=True)
         lf.real_par_controller.makeCalculator()
     
-class NewQ(TestCase):
-    aln = LoadSeqs(data={
-    'seq1': 'TGTGGCACAAATACTCATGCCAGCTCATTACAGCATGAGAACAGCAGTTTATTACTCACT',
-    'seq2': 'TGTGGCACAAATACTCATGCCAGCTCATTACAGCATGAGAACAGCAGTTTATTACTCACT'},
-    moltype=DNA)
-    tree = LoadTree(tip_names=['seq1', 'seq2'])
-    
-    asymm_nuc_probs_GC = dict(A=0.1,T=0.1,C=0.4,G=0.4)
-    asymm_root_probs = _root_probs(asymm_nuc_probs_GC)
-    symm_nuc_probs = dict(A=0.25,T=0.25,C=0.25,G=0.25)
-    symm_root_probs = _root_probs(symm_nuc_probs)
-    
-    def test_newQ_is_nuc_process(self):
-        """newQ is an extension of an independent nucleotide process"""
-        nuc = Nucleotide(motif_probs = self.asymm_nuc_probs_GC)
-        new_di = Nucleotide(motif_length=2, mprob_model='monomer',
-            motif_probs = self.asymm_nuc_probs_GC)
-        
-        nuc_lf = nuc.makeLikelihoodFunction(self.tree)
-        new_di_lf = new_di.makeLikelihoodFunction(self.tree)
-        # newQ branch length is exactly motif_length*nuc branch length
-        nuc_lf.setParamRule('length', is_independent=False, init=0.2)
-        new_di_lf.setParamRule('length', is_independent=False, init=0.4)
-        
-        nuc_lf.setAlignment(self.aln)
-        new_di_lf.setAlignment(self.aln)
-        self.assertFloatEqual(nuc_lf.getLogLikelihood(),
-                                new_di_lf.getLogLikelihood())
-    
-    def test_newQ_is_not_oldQ(self):
-        """newQ produces a likelihood different to oldQ when monomers are
-        not equi-frequent"""
-        new_di = Nucleotide(motif_length=2, mprob_model='monomer',
-                                motif_probs = self.asymm_nuc_probs_GC)
-        old_di = Nucleotide(motif_length=2, mprob_model=None,
-                                motif_probs = self.asymm_root_probs)
-        
-        new_di_lf = new_di.makeLikelihoodFunction(self.tree)
-        old_di_lf = old_di.makeLikelihoodFunction(self.tree)
-        
-        new_di_lf.setParamRule('length', is_independent=False, init=0.4)
-        old_di_lf.setParamRule('length', is_independent=False, init=0.4)
-        
-        new_di_lf.setAlignment(self.aln)
-        old_di_lf.setAlignment(self.aln)
-        self.failIfAlmostEqual(new_di_lf.getLogLikelihood(),
-                               old_di_lf.getLogLikelihood(), places=2)
-    
-    def test_newQ_eq_oldQ(self):
-        """when monomers are equi-frequent, new and old Q are the same"""
-        new_di = Nucleotide(motif_length=2, mprob_model='monomer',
-                                motif_probs = self.symm_nuc_probs)
-        old_di = Nucleotide(motif_length=2, mprob_model=None,
-                                motif_probs = self.symm_root_probs)
-        
-        new_di_lf = new_di.makeLikelihoodFunction(self.tree)
-        old_di_lf = old_di.makeLikelihoodFunction(self.tree)
-        
-        new_di_lf.setParamRule('length', is_independent=False, init=0.4)
-        old_di_lf.setParamRule('length', is_independent=False, init=0.4)
-        
-        new_di_lf.setAlignment(self.aln)
-        old_di_lf.setAlignment(self.aln)
-        self.assertFloatEqual(old_di_lf.getLogLikelihood(),
-                                new_di_lf.getLogLikelihood())
-    
-    def test_nuc_models_unaffected(self):
-        """nuc models should not be affected by the mprob_model arg"""
-        gtr = [MotifChange(x,y) for x,y in 'AC AG AT CG CT'.split()]
-        mg_gtr = Nucleotide(predicates=gtr, mprob_model='monomer')
-        gy_gtr = Nucleotide(predicates=gtr, mprob_model=None)
-        mg_gtr = mg_gtr.makeLikelihoodFunction(self.tree)
-        gy_gtr = gy_gtr.makeLikelihoodFunction(self.tree)
-        for param, val in zip('A/C A/G A/T C/G C/T'.split(), [.1,2.,.5,.5,3.]):
-            for lf in mg_gtr, gy_gtr:
-                lf.setParamRule(param, value=val)
-        mg_gtr.setAlignment(self.aln)
-        gy_gtr.setAlignment(self.aln)
-        self.assertFloatEqual(mg_gtr.getLogLikelihood(), gy_gtr.getLogLikelihood())
-    
-    def test_position_specific_mprobs(self):
-        """correctly compute likelihood when positions have distinct
-        probabilities"""
-        aln_len = len(self.aln)
-        posn1 = []
-        posn2 = []
-        for name, seq in self.aln.todict().items():
-            p1 = [seq[i] for i in range(0,aln_len,2)]
-            p2 = [seq[i] for i in range(1,aln_len,2)]
-            posn1.append([name, ''.join(p1)])
-            posn2.append([name, ''.join(p2)])
-        
-        # the position specific alignments
-        posn1 = LoadSeqs(data=posn1)
-        posn2 = LoadSeqs(data=posn2)
-        
-        # NOTE: I had to set do_scaling=False to get this to work
-        # NOTE: the argument word_length will be deprecated in a future release
-        sm = Nucleotide(word_length=2, do_scaling=False) # a newQ dinucleotide model
-        lf = sm.makeLikelihoodFunction(self.tree)
-        lf.setAlignment(posn1)
-        posn1_lnL = lf.getLogLikelihood()
-        lf.setAlignment(posn2)
-        posn2_lnL = lf.getLogLikelihood()
-        expect_lnL = posn1_lnL+posn2_lnL
-        # the joint model
-        lf.setAlignment(self.aln)
-        aln_lnL = lf.getLogLikelihood()
-        
-        # setting the full alignment, which has different motif probs, should
-        # produce a different lnL
-        self.failIfAlmostEqual(expect_lnL, aln_lnL)
-        
-        # the position specific model
-        
-        # set the arguments for taking position specific mprobs
-        sm = Nucleotide(word_length=2, mprob_model='monomers', do_scaling=False)
-        lf = sm.makeLikelihoodFunction(self.tree)
-        lf.setAlignment(self.aln)
-        posn12_lnL = lf.getLogLikelihood()
-        self.assertFloatEqual(expect_lnL, posn12_lnL)
-    
-    def test_getting_node_mprobs(self):
-        """return correct motif probability vector for tree nodes"""
-        tree = LoadTree(treestring='(a:.2,b:.2,(c:.1,d:.1):.1)')
-        aln = LoadSeqs(data={
-        'a': 'TGTG',
-        'b': 'TGTG',
-        'c': 'TGTG',
-        'd': 'TGTG',
-        })
-        
-        motifs = ['T', 'C', 'A', 'G']
-        aX = MotifChange(motifs[0], motifs[3], forward_only=True).aliased('aX')
-        bX = MotifChange(motifs[3], motifs[0], forward_only=True).aliased('bX')
-        edX = MotifChange(motifs[1], motifs[2], forward_only=True).aliased('edX')
-        cX = MotifChange(motifs[2], motifs[1], forward_only=True).aliased('cX')
-        sm = Nucleotide(predicates=[aX, bX, edX, cX], equal_motif_probs=True)
-        
-        lf = sm.makeLikelihoodFunction(tree)
-        lf.setParamRule('aX', edge='a', value=8.0)
-        lf.setParamRule('bX', edge='b', value=8.0)
-        lf.setParamRule('edX', edge='edge.0', value=2.0)
-        lf.setParamRule('cX', edge='c', value=0.5)
-        lf.setParamRule('edX', edge='d', value=4.0)
-        lf.setAlignment(aln)
-        
-        # we construct the hand calc variants
-        mprobs = ones(4, float) * .25
-        a = make_p(.2, (0,3), 8)
-        a = dot(mprobs, a)
-        
-        b = make_p(.2, (3, 0), 8)
-        b = dot(mprobs, b)
-        
-        e = make_p(.1, (1, 2), 2)
-        e = dot(mprobs, e)
-        
-        c = make_p(.1, (2, 1), 0.5)
-        c = dot(e, c)
-        
-        d = make_p(.1, (1, 2), 4)
-        d = dot(e, d)
-        
-        prob_vectors = lf.getMotifProbsByNode()
-        self.assertFloatEqual(prob_vectors['a'].array, a)
-        self.assertFloatEqual(prob_vectors['b'].array, b)
-        self.assertFloatEqual(prob_vectors['c'].array, c)
-        self.assertFloatEqual(prob_vectors['d'].array, d)
-        self.assertFloatEqual(prob_vectors['edge.0'].array, e)
-    
-    def test_get_statistics(self):
-        """get statistics should correctly apply arguments"""
-        for mprob_model in None, 'monomer', 'monomers':
-            di = Nucleotide(motif_length=2, mprob_model=mprob_model)
-            lf = di.makeLikelihoodFunction(self.tree)
-            for wm, wt in [(True, True), (True, False), (False, True),
-                           (False, False)]:
-                stats = lf.getStatistics(with_motif_probs=wm, with_titles=wt)
-        
-    def test_set_tables_format(self):
-        """should correctly modify the format of tables displayed by str(lf)"""
-        sm = Nucleotide()
-        lf = sm.makeLikelihoodFunction(self.tree)
-        lf.setTablesFormat(space=2,digits=2)
-        stats = lf.getStatistics(with_motif_probs=False, with_titles=False)
-        expected=['====================',
-                  'edge  parent  length',
-                  '--------------------',
-                  'seq1    root    1.00',
-                  'seq2    root    1.00',
-                  '--------------------']
-        self.assertEqual(str(stats[0]),'\n'.join(expected))
-
 
 if __name__ == '__main__':
     main()
