@@ -4,14 +4,15 @@ __all__ = ['test_geometry',
            'test_matrix_logarithm',
            'test_optimisers',
            'test_stats',
-           'test_unifrac']
+           'test_unifrac',
+           'test_spatial']
 
 __author__ = ""
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Rob Knight", "Peter Maxwell", "Catherine Lozupone",
-                    "Gavin Huttley", "Sandra Smit"]
+               "Gavin Huttley", "Sandra Smit", "Marcin Cieslik"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
