@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-__all__ = ['test_aaindex', 
+__all__ = ['test_aaindex',
            'test_agilent_microarray',
            'test_blast',
            'test_bpseq',
@@ -16,23 +16,25 @@ __all__ = ['test_aaindex',
            'test_ncbi_taxonomy',
            'test_nexus',
            'test_phylip',
-           'test_record', 
+           'test_record',
            'test_record_finder',
            'test_rfam',
            'test_rna_fold',
            'test_rnaview',
            'test_sprinzl',
            'test_tree',
-           'test_unigene']
+           'test_unigene'
+           'test_stride']
 
 __author__ = ""
 __copyright__ = "Copyright 2007-2009, The Cogent Project"
 __credits__ = ["Jeremy Widmann", "Catherine Lozuopone", "Gavin Huttley",
                     "Rob Knight", "Sandra Smit", "Micah Hamady",
                     "Jeremy Widmann", "Hua Ying", "Greg Caporaso",
-                    "Zongzhi Liu", "Jason Carnes", "Peter Maxwell"]
+                    "Zongzhi Liu", "Jason Carnes", "Peter Maxwell",
+                    "Marcin Cieslik"]
 __license__ = "GPL"
-__version__ = "1.3.1"
+__version__ = "1.4"
 __maintainer__ = "Rob Knight"
 __email__ = "rob@spot.colorado.edu"
 __status__ = "Production"
